//
//  AppDelegate.swift
//  N80X 1.83A
//
//  Copyright (C) 2015-2024 Hiroshi Kanemoto.
//	Released under the MIT license
//	http://opensource.org/licenses/mit-license.php
//
//  Created by hiroshi on 2016/04/04.
//  Last Modified         2024/05/19
//
//  お察しください、、、：
//  ツッコミどころ満載のソースではありますが、macOSネイティブでない日曜プログラマー
//  故にご容赦ください。ここはこうした方がいいよとかアドバイスがあればご教示頂ければ
//  幸いです。かなりUnsafeな内容となっております。
//
//  ご覧の通り、swift/Objective-C/C の混成構成となっております。
//
//  追伸：当方、Intel Macしか所有しておりませんので、ARM系での動作は確認しており
//       ません。悪しからずご了承ください。
//
//                                              retro8bits@gmail.com
//
import Cocoa
import GameController

@main
class AppDelegate: NSObject, NSApplicationDelegate {
    fileprivate var bmpimg:DblBuffBmpImage! = nil
    fileprivate let keyboard:GCKeyboard! = nil
    fileprivate var n80:n80em! = nil
    fileprivate var config:n80config_data! = nil
    public var callback: (() -> Bool)? = nil
    //-------------------------------------------------------------------------
    //  Properties
    //-------------------------------------------------------------------------
    internal var BmpImg:DblBuffBmpImage {
        return self.bmpimg
    }
    internal var N80:n80em {
        return self.n80
    }
    //-------------------------------------------------------------------------
    //  Initialize N80 Emulator object
    //-------------------------------------------------------------------------
    public func req_init_n80() {
        self.config = n80config_data();
        self.get_configure()

        self.bmpimg = DblBuffBmpImage(640, 400)
        self.n80 = n80em(self.bmpimg, data: self.config)
    }
    //-------------------------------------------------------------------------
    //  Config Access
    //-------------------------------------------------------------------------
    private func get_configure() {
        let userdef = UserDefaults.standard
        userdef.register(defaults: ["interlace": Bool(true)])
        userdef.register(defaults: ["hires_crt": Bool(false)])
        userdef.register(defaults: ["tenkeyless": Bool(true)])
        userdef.register(defaults: ["chiqlappe": Bool(false)])
        userdef.register(defaults: ["nowait": Bool(false)])
        userdef.register(defaults: ["n80_n80_file_name" : "PC-8001.ROM"])
        userdef.register(defaults: ["n80_4th_file_name" : ""])
        userdef.register(defaults: ["n80_fdu_file_name" : "PDISK.ROM"])
        
        self.config.interlace = userdef.bool(forKey: "interlace")
        self.config.hires_crt = userdef.bool(forKey: "hires_crt")
        self.config.tenkeyless = userdef.bool(forKey: "tenkeyless")
        self.config.chiqlappe = userdef.bool(forKey: "chiqlappe")
        self.config.nowait = userdef.bool(forKey: "nowait")
        self.set_string_slot(value: userdef.string(forKey: "n80_n80_file_name")!, slot:0)
        self.set_string_slot(value: userdef.string(forKey: "n80_4th_file_name")!, slot:1)
        self.set_string_slot(value: userdef.string(forKey: "n80_fdu_file_name")!, slot:2)
    }
    
    private func set_configure() {
        let userdef = UserDefaults.standard

        userdef.set(self.config.interlace, forKey: "interlace")
        userdef.set(self.config.hires_crt, forKey: "hires_crt")
        userdef.set(self.config.tenkeyless, forKey: "tenkeyless")
        userdef.set(self.config.chiqlappe, forKey: "chiqlappe")
        userdef.set(self.config.nowait, forKey: "nowait")

        let slot0 = get_string_slot(slot: 0)
        userdef.set(slot0, forKey: "n80_n80_file_name")
        let slot1 = get_string_slot(slot: 1)
        userdef.set(slot1, forKey: "n80_4th_file_name")
        let slot2 = get_string_slot(slot: 2)
        userdef.set(slot2, forKey: "n80_fdu_file_name")
        //userdef.synchronize()
    }
    
    private func set_string_slot(value:String, slot:Int32) {
        var cnt:Int32 = 0
        let dat = value.data(using: String.Encoding.shiftJIS, allowLossyConversion: true)
        dat?.withUnsafeBytes { chunk in
            for byte in chunk.bindMemory(to: Int8.self) {
                self.config.set_char(byte, index:cnt, slot:slot)
                cnt += 1
            }
        }
    }
    func get_string_slot(slot:Int32) -> String {
        let cnt = Int(self.config.get_size(slot))
        var dat = Data(count:cnt)
        dat.withUnsafeMutableBytes{ (p: UnsafeMutableRawBufferPointer) -> Void in
            for i in 0 ..< cnt {
                let index = Int32(i)
                let c = self.config.get_char(index, slot: slot)
                p[i] = UnsafeMutableRawBufferPointer.Element(CChar(c))
            }
        }
        return String(data: dat, encoding: String.Encoding.shiftJIS) ?? ""
    }
    //-------------------------------------------------------------------------
    //  AppDelegate Event
    //-------------------------------------------------------------------------
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        // Insert code here to initialize your application
    }

    func applicationWillTerminate(_ aNotification: Notification) {
        // Insert code here to tear down your application
        self.set_configure()
        self.n80.req_dispose()
    }
    //-------------------------------------------------------------------------
    //  Main Menu bar
    //-------------------------------------------------------------------------
    @IBOutlet weak var menubar: NSMenu!
    @IBOutlet weak var menuPCG: NSMenuItem!
    @IBOutlet weak var menuTenKeyLess: NSMenuItem!
    @IBOutlet weak var menuInterlace: NSMenuItem!
    @IBOutlet weak var menuNewPCG: NSMenuItem!
    @IBOutlet weak var menuDrive1: NSMenuItem!
    @IBOutlet weak var menuDrive2: NSMenuItem!
    @IBOutlet weak var menuEmu: NSMenuItem!
    @IBOutlet weak var menuFile: NSMenuItem!
    
    @IBOutlet weak var menuOpenDiskRom: NSMenuItem!
    @IBAction func menuTenkeyLess(_ sender: Any) {
        self.n80.n80kb.toggle_tenkeyless()
    }
    
    @IBAction func menuQuit(_ sender: Any) {
        NSApplication.shared.terminate(self)
    }
  
    @IBAction func menuReset(_ sender: Any) {
        self.n80.req_reset()
    }
    
    @IBAction func menuConfig(_ sender: Any) {
        self.n80.req_enter_config()
    }
    
    @IBAction func menuPCG(_ sender: Any) {
        self.n80.req_toggle_pcg()
    }
    
    @IBAction func menuNewPCG(_ sender: Any) {
        n80device.param.use_new_pcg = !n80device.param.use_new_pcg
        self.n80.n80cfg.reflect_menu_state()
    }
    
    @IBAction func menuDisk1(_ sender: NSMenuItem) {
        self.open_d88_image_file(unit:0, sender:sender)
    }
    @IBAction func menuDisk2(_ sender: NSMenuItem) {
        self.open_d88_image_file(unit:1, sender:sender)
    }
    
    @IBAction func menuInterlace(_ sender: NSMenuItem) {
        n80device.param.interlace = !n80device.param.interlace
        self.n80.n80gr.chg_palette(n80device.param.interlace)
        self.n80.n80cfg.reflect_menu_state()
    }
    
    @IBAction func menuOpen(_ sender: Any) {
        if self.n80.main_rom_ready {
            self.n80.req_open_image_file()
        } else if (self.callback?() ?? false) {
            self.menuEmu.isEnabled = true
            if !n80device.fdu.rom_ready {
                self.menuOpenDiskRom.isHidden = false;
                self.menuDrive1.isHidden = true
                self.menuDrive2.isHidden = true
            }
        }
    }
    
    @IBAction func menuOpenDiskROM(_ sender: Any) {
        if self.n80.req_load_disk_rom() {
            self.menuOpenDiskRom.isHidden = true;
            self.menuDrive1.isHidden = false
            self.menuDrive2.isHidden = false
        }
    }

    public func open_d88_image_file(unit:Int, sender:NSMenuItem) {
        if sender.state == NSControl.StateValue.on {
            sender.state = NSControl.StateValue.off
            self.n80.req_eject(Int32(unit))
        } else {
            if self.n80.req_open_d88_file(Int32(unit)) {
                sender.state = NSControl.StateValue.on
            }
        }
    }
    
}

