//
//  DblBuffBmpImage.m
//  N80X
//
//  Created by hiroshi on 2021/08/20.
//

#import <Foundation/Foundation.h>
#import "DblBuffBmpImage.h"
#import <OpenGL/OpenGL.h>

@implementation DblBuffBmpImage

-(id)init:(int) p_width : (int) p_height {
    if (self = [super init]) {
        self.width = p_width;
        self.height = p_height;
        self->_bmp1 = malloc(p_width * p_height * 4);
        self->_bmp2 = malloc(p_width * p_height * 4);
        //self.bpp = GL_UNSIGNED_SHORT_4_4_4_4;
        //self.bpp = GL_UNSIGNED_BYTE;
        self->_bmpctx1 = [self createBitmapContext:p_width :p_height :self->_bmp1];
        self->_bmpctx2 = [self createBitmapContext:p_width :p_height :self->_bmp2];
        self->_flip = true;
        self->_lock = [[NSRecursiveLock alloc] init];
    }
    return self;
}
-(const int)getBytesPerRow {
    return self->_width * 4;
}
-(const GLvoid *)getBitmap {
    if (self->_flip) {
        return self->_bmp1;
    } else {
        return self->_bmp2;
    }
}

-(const GLvoid *)getSecondBitmap {
    [self->_lock lock];
    @try {
        if (!self->_flip) {
            return self->_bmp1;
        } else {
            return self->_bmp2;
        }
    }
    @finally {
        [self->_lock unlock];
    }
}

-(CGContextRef) getBmpCtx {
    [self->_lock lock];
    @try {
        if (self->_flip) {
            return self->_bmpctx1;
        } else {
            return self->_bmpctx2;
        }
    }
    @finally {
        [self->_lock unlock];
    }
}
-(CGContextRef) getSecondBmpCtx {
    if (!self->_flip) {
        return self->_bmpctx1;
    } else {
        return self->_bmpctx2;
    }
}

-(void)flip {
    [self->_lock lock];
    self->_flip = !self->_flip;
    [self->_lock unlock];
}

-(void)lock {
    [self->_lock lock];
}

-(void)unlock {
    [self->_lock unlock];
}
-(CGContextRef)createBitmapContext:(int)pixelsWide : (int)pixelsHigh : (void *)bitmap {
    CGColorSpaceRef    colorSpace;
    int                bitmapByteCount;
    int                bitmapBytesPerRow;
    
    bitmapBytesPerRow   = (pixelsWide * 4);
    bitmapByteCount     = (bitmapBytesPerRow * pixelsHigh);
    colorSpace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
    return CGBitmapContextCreate (bitmap,
                                  pixelsWide,
                                  pixelsHigh,
                                  8,    // bits per component (r/g/b)
                                  bitmapBytesPerRow,
                                  colorSpace,
                                  //kCGImageAlphaLast);
                                  kCGImageAlphaNoneSkipLast);
}

-(void)setPixel:(BOOL)fore X:(int)x Y:(int)y R:(UInt8)r G:(UInt8)g B:(UInt8)b {
    if (x >= 0 && x < self.width && y >= 0 && y < self.height) {
        int offset = (x * 4) + (4 * 640 * y);
        UInt8 *ptr = (UInt8 *)(fore ? self.bitmap : self.bitmap_back);
        ptr[offset] = r;
        ptr[offset+1] = g;
        ptr[offset+2] = b;
        ptr[offset+3] = 0xff;
    }
}

-(RGBPixel*)getPixel:(BOOL)fore X:(int)x Y:(int)y {
    if (x >= 0 && x < self.width && y >= 0 && y < self.height) {
        int offset = (x * 4) + (4 * 640 * y);
        UInt8 *ptr = (UInt8 *)(fore ? self.bitmap : self.bitmap_back);
        return (RGBPixel*) &ptr[offset];
    }
    return NULL;
}

-(void)free {
    if (self->_bmp1 != NULL) {
        free(self->_bmp1);
        self->_bmp1 = NULL;
    }
    if (self->_bmp2 != NULL) {
        free(self->_bmp2);
        self->_bmp2 = NULL;
    }
    if (self->_bmpctx1 != NULL) {
        CGContextRelease(self->_bmpctx1);
        self->_bmpctx1 = NULL;
    }
    if (self->_bmpctx2 != NULL) {
        CGContextRelease(self->_bmpctx2);
        self->_bmpctx2 = NULL;
    }
}
-(void)dealloc {
    NSLog(@"#### DblBuffBmpImage.dealloc ####");
    [self free];
}

@end

