/**==========================================================================

	PC-8031 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2024 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	last modified  Apr,27,2024.

;==========================================================================*/
#include "../PC-8001/PC-8001.h"
#ifdef __cplusplus
extern "C"
{
#endif
#define export
/**--------------------------------------------------------------------------
 **		METHODS
 **------------------------------------------------------------------------*/
static BOOL _fdc_command_load_cmn(LPFDUCOMMAND self) {
	if (self->count < 2)
		return FALSE;
	self->MT = self->commands[0] & 0x80 ? 1 : 0;
	self->MF = self->commands[0] & 0x40 ? 1 : 0;
	self->SK = self->commands[0] & 0x20 ? 1 : 0;
	self->HD = self->commands[1] & 0x04 ? 1 : 0;
	self->US = self->commands[1] & 0x03;
	return FALSE;
}
static BOOL _fdc_command_load_std(LPFDUCOMMAND self) {
	if (self->count < 9)
		return FALSE;
	self->load_cmn(self);
	self->id.C	= self->commands[2];
	self->id.H	= self->commands[3];
	self->id.R	= self->commands[4];
	self->id.N	= self->commands[5];
	self->EOT	= self->commands[6];
	self->GPL	= self->commands[7];
	self->DTL	= self->commands[8];
	self->STP	= self->DTL;
	return TRUE;
}
static BOOL _fdc_command_load_write_id(LPFDUCOMMAND self) {
	if (self->count != 6)
		return FALSE;
	self->load_cmn(self);
	self->id.C	= 0;
	self->id.H	= 0;
	self->id.R	= 0;
	self->id.N	= self->commands[2];
	self->SC	= self->commands[3];
	self->GPL	= self->commands[4];
	self->D		= self->commands[5];
	return TRUE;
}
static BOOL _fdc_command_load_specify(LPFDUCOMMAND self) {
	if (self->count != 3)
		return FALSE;
	self->SRT = ((self->commands[1] >> 4) & 0x0f);
	self->HUT = (self->commands[1] & 0x0f);
	self->HLT = ((self->commands[2] >> 1) & 0x7f);
	self->ND  = (self->commands[2] & 0x01);
	return TRUE;
}
static BOOL _fdc_command_load_seek(LPFDUCOMMAND self) {
	if (self->count != 3)
		return FALSE;
	self->load_cmn(self);
	self->NCN = self->commands[2];
	return TRUE;
}
static void _fdc_command_clear(LPFDUCOMMAND self) {
	self->count = 0;
	self->command_code = 0;
}
static BYTE _fdc_command_push0(LPFDUCOMMAND self, BYTE data) {
	self->clear(self);
	self->command_code = data & 0x1f;
	self->commands[self->count++] = data;
	return self->command_code;
}
static int _fdc_command_push(LPFDUCOMMAND self, BYTE data) {
	self->commands[self->count++] = data;
	return self->count;
}
export void _fdc_init_command(LPFDUCOMMAND self) {
	self->clear		= _fdc_command_clear;
	self->push0		= _fdc_command_push0;
	self->push		= _fdc_command_push;
	self->load_cmn	= _fdc_command_load_cmn;
	self->load_std	= _fdc_command_load_std;
	self->load_wid	= _fdc_command_load_write_id;
	self->load_spec = _fdc_command_load_specify;
	self->load_seek = _fdc_command_load_seek;
	self->clear(self);
}
#ifdef __cplusplus
}	// extern "C"
#endif
