/**==========================================================================

	PC-8031 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2024 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	last modified  Apr,27,2024.

;==========================================================================*/
#include "../PC-8001/PC-8001.h"
#ifdef __cplusplus
extern "C"
{
#endif
/**--------------------------------------------------------------------------
 **		ASM Modules
 **------------------------------------------------------------------------*/
static BYTE __cdecl _fdu_mbread(LPFDUMEM self, USHORT Adr, DWORD *W) {
	BYTE mb = 0xff;
	
	switch (Adr & 0xf000) {
		case 0x0000:
		case 0x1000:
			return self->_rom[Adr];
		case 0x2000:
		case 0x3000:
			return 0xff;
		case 0x4000:
		case 0x5000:
		case 0x6000:
		case 0x7000:
			return self->_ram[Adr & 0x3fff];
		default:
#if _DEBUG
			n80device.logger("FDU: [MEMORY] invalid read memory access\n");
#endif
			return self->_ram[Adr & 0x3fff];
			break;
	}
	return mb;
}
    
static void __cdecl _fdu_mbwrite(LPFDUMEM self, USHORT Adr, BYTE v, DWORD *W) {
	switch (Adr & 0xf000) {
		case 0x0000:
		case 0x1000:
#if _DEBUG
			n80device.logger("FDU: [MEMORY] invalid write to ROM area access [%4.4x <= %2.2X]\n", Adr, v);
#endif
			break;
		case 0x4000:
		case 0x5000:
		case 0x6000:
		case 0x7000:
			self->_ram[Adr & 0x3fff] = v;
			break;
		default:
#if _DEBUG
			n80device.logger("FDU: [MEMORY] invalid write memory access\n");
#endif
			self->_ram[Adr & 0x3fff] = v;
			break;
	}
}

static void _fdu_clear_memory(LPFDUMEM self) {
	LPBYTE	mem = self->_ram;
	int  i;

	for (i = 0; i < 0x2000; ++i) {
		*mem++ = 0x00;
		*mem++ = 0xff;
	}
}

export void _fdu_init_memory(void) {
	n80device.fdu.mem.read  = _fdu_mbread;
	n80device.fdu.mem.write = _fdu_mbwrite;
	n80device.fdu.mem.clear = _fdu_clear_memory;
}
#ifdef __cplusplus
}	// extern "C"
#endif
