/**==========================================================================

	PC-8031 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2024 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	last modified  Apr,27,2024.

;==========================================================================*/
#include "../PC-8001/PC-8001.h"
#ifdef __cplusplus
extern "C"
{
#endif
#define export
static const char* err_size = "Described size exceeds actual image length\n";
static const char* err_head = "Header size is shorter than specified\n";
static const char* err_trkz = "Track data exceeds image length\n";
static const char* err_sect = "Sector data exceeds image length\n";
static const char* err_scnt = "Amount of Sectors is over 32\n";
static const char* err_id_c = "wrong cylinder id\n";
static const char* err_id_h = "wrong head id\n";
static const char* err_id_r = "wrong sector id\n";
/**--------------------------------------------------------------------------
 **		METHODS
 **------------------------------------------------------------------------*/
#ifndef __BIG_ENDIAN__
USHORT read_short(LPBYTE ptr) {
	USHORT v = 0;
	v = (USHORT)ptr[1];
	v <<= 8;
	v += (USHORT)ptr[0];
	return v;
}
DWORD read_quad(LPBYTE ptr) {
	DWORD v = 0;
	v = (DWORD)ptr[3];
	v <<= 8;
	v += (DWORD)ptr[2];
	v <<= 8;
	v += (DWORD)ptr[1];
	v <<= 8;
	v += (DWORD)ptr[0];
	return v;
}
#else
USHORT read_short(LPBYTE ptr) {
	USHORT v = 0;
	v = (USHORT)ptr[0];
	v <<= 8;
	v += (USHORT)ptr[1];
	return v;
}
DWORD read_quad(LPBYTE ptr) {
	DWORD v = 0;
	v = (DWORD)ptr[0];
	v <<= 8;
	v += (DWORD)ptr[1];
	v <<= 8;
	v += (DWORD)ptr[2];
	v <<= 8;
	v += (DWORD)ptr[3];
	return v;
}
#endif
static BOOL _media_check_image(LPFLOPPYDISK self) {
	self->header = (LPD88HEADER)self->image_raw_data;
	// 記載されているｻｲｽﾞが実際のｲﾒｰｼﾞ長を超過している場合はｴﾗｰ
	if (read_quad(self->header->disksize) > self->image_length) {
		self->err_msg = err_size;
		return FALSE;
	}
	// ﾍｯﾀﾞｰ領域がｲﾒｰｼﾞ長を超過している場合はｴﾗｰ
	if (self->image_length <= sizeof(D88HEADER)) {
		self->err_msg = err_head;
		return FALSE;
	}
	// ﾄﾗｯｸﾁｪｯｸ
	for (int t = 0; t < 164; ++t) {
		BYTE C = (BYTE)(t / 2);	// cylinder
		BYTE H = (BYTE)(t % 2);	// head

		DWORD offset = read_quad(&self->header->trackptr[t * 4]);
		if (offset == 0)
			break;
		// ｾｸﾀﾍｯﾀﾞｰ領域がｲﾒｰｼﾞ長を超過している場合はｴﾗｰ
		if ((offset + 16) > self->image_length) {
			self->err_msg = err_trkz;
#if _DEBUG
			n80device.logger(self->err_msg);
#endif
			return FALSE;
		}
		LPD88SECTOR track = (LPD88SECTOR)&self->image_raw_data[offset];
		USHORT sectors = read_short(track->sectors);
		if (sectors > 32) {
			self->err_msg = err_scnt;
#if _DEBUG
			n80device.logger(self->err_msg);
#endif
			return FALSE;
		}
		for (int s = 0; s < sectors; ++s) {
			BYTE R = (BYTE)(s + 1);
			USHORT sectlen = read_short(track->length);
			offset += sectlen + 16;
			// ｾｸﾀﾃﾞｰﾀ領域がｲﾒｰｼﾞ長を超過している場合はｴﾗｰ
			if (offset > self->image_length) {
				self->err_msg = err_sect;
#if _DEBUG
				n80device.logger(self->err_msg);
#endif
				//return FALSE;
			}
			if (track->id.C != C) {
				self->err_msg = err_id_c;
#if _DEBUG
				n80device.logger(self->err_msg);
#endif
				//return FALSE;
			}
			if (track->id.H != H) {
				self->err_msg = err_id_h;
#if _DEBUG
				n80device.logger(self->err_msg);
#endif
				//return FALSE;
			}
			if (track->id.R != R) {
				self->err_msg = err_id_r;
#if _DEBUG
				n80device.logger(self->err_msg);
#endif
				//return FALSE;
			}
			track = (LPD88SECTOR)&self->image_raw_data[offset];
		}
	}
	{	// 初代PC-8801は1DのFDが読めない
		int C = 0, H = 0, R = 1;
		DWORD direct_offset = read_quad(&self->header->trackptr[0]);//0x2B0;
		for (int t = 0; t < 164; ++t) {
			//DWORD offset = read_quad(&self->header->trackptr[t * 4]);
			if ((direct_offset + 16) > self->image_length)
				break;
			LPD88SECTOR track = (LPD88SECTOR)&self->image_raw_data[direct_offset];
			USHORT sectors = read_short(track->sectors);
			//DWORD offset = read_quad(&self->header->trackptr[t * 4]);
			//USHORT sectors = 0;
			//if (offset > 0) {
			//	if ((direct_offset + 16) > self->image_length)
			//		break;
			//	track = (LPD88SECTOR)&self->image_raw_data[direct_offset];
			//	sectors = read_short(track->sectors);
			//}
			for (int s = 0; s < sectors; ++s) {
				if ((direct_offset + 16) > self->image_length)
					break;
				track = (LPD88SECTOR)&self->image_raw_data[direct_offset];
				USHORT sectlen = read_short(track->length);
				if ((direct_offset + 16 + sectlen) > self->image_length)
					break;
#if _DEBUG
				n80device.logger("MEDIA: [ID] C:%d, H:%d, R:%d, N:%d =>", track->id.C, track->id.H, track->id.R, track->id.N);
#endif
				// convert 1D sector id to 2D id
				if (self->header->disktype == 0x30 /* 1D */) {
					track->id.H = track->id.C % 2;
					track->id.C = track->id.C / 2;
					//track->id.C = (BYTE)C;
					//track->id.H = (BYTE)H;
					//track->id.R = (BYTE)R;
#if _DEBUG
					n80device.logger(" {%d, %d, %d}\n", track->id.C, track->id.H, track->id.R);
#endif
				}
				direct_offset += (16 + sectlen);
				++R;
			}
			R = 1;
			if (++H > 1) {
				++C;
				H = 0;
			}
		}
	}
	return TRUE;
}

static LPD88SECTOR _d88_find_sector_by_index(LPFLOPPYDISK self, LPSECTORID id) {
	int C = id->C, H = id->H;
	int index = ((C * 2) + H) * 4;
	DWORD offset = read_quad(&self->header->trackptr[index]);
	if (offset == 0) {
		return NULL;
	}
	LPD88SECTOR track = (LPD88SECTOR)&self->image_raw_data[offset];
	USHORT sectors = read_short(track->sectors);
	for (int s = 0; s < sectors; ++s) {
		BYTE R = (BYTE)(s + 1);
		if (id->R == R)
			return track;
		USHORT sectlen = read_short(track->length);
		offset += sectlen + 16;
	}
	return NULL;
}
static LPD88SECTOR _d88_find_sector_by_all_scan(LPFLOPPYDISK self, LPSECTORID id) {
	DWORD direct_offset = read_quad(&self->header->trackptr[0]);
	for (int t = 0; t < 164; ++t) {
		if ((direct_offset + 16) > self->image_length)
			break;
		LPD88SECTOR track = (LPD88SECTOR)&self->image_raw_data[direct_offset];
		USHORT sectors = read_short(track->sectors);
		for (int s = 0; s < sectors; ++s) {
			if ((direct_offset + 16) > self->image_length)
				break;
			track = (LPD88SECTOR)&self->image_raw_data[direct_offset];
			USHORT sectlen = read_short(track->length);
			if ((direct_offset + 16 + sectlen) > self->image_length)
				break;
#if _DEBUG
			//n80device.logger("MEDIA: [ID] C:%d, H:%d, R:%d, N:%d\n", track->id.C, track->id.H, track->id.R);
#endif
			if (track->id.C == id->C && track->id.H == id->H && track->id.R == id->R)
				return track;
			direct_offset += (16 + sectlen);
		}
	}
	return NULL;
}

static LPD88SECTOR_EX _d88_find_sector(LPFLOPPYDISK self, LPSECTORID id) {
	//LPD88SECTOR sector = _d88_find_sector_by_index(self, id);
	//if (sector == NULL)
	//	sector = _d88_find_sector_by_all_scan(self, id);
	LPD88SECTOR	sector = _d88_find_sector_by_all_scan(self, id);
	return (LPD88SECTOR_EX)sector;
}

export BOOL _fdu_init_media(LPFLOPPYDISK self, LPFDU fdu) {
	self->find_sector = _d88_find_sector;
	return _media_check_image(self);
}
#ifdef __cplusplus
}	// extern "C"
#endif
