/**==========================================================================

	PC-8031 Device Emulator for Win32 Ver 0.90
	Copyright (C)1993-2024 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	last modified  Apr,27,2024.

;==========================================================================*/
#ifndef _PC_8031_H_
#define _PC_8031_H_
#include "../z80/z80.h"
#ifdef __APPLE__
#ifndef TRUE
#define TRUE            true
#endif
#ifndef FALSE
#define FALSE           false
#endif
#endif
#ifdef __cplusplus
extern "C"
{
#endif
#define export
#define trkdiff(a, b)	(((a > b) ? (a - b) : (b - a)) + 1)
/**--------------------------------------------------------------------------
 **		定数定義
 **------------------------------------------------------------------------*/
#ifdef _MSC_VER
#pragma region PD765A status
#else
 // #region Z80 registers
#endif
// FDC Main Status Register bits
#define FDC_ST_D0B				(1)				// FDD 0 Busy
#define FDC_ST_D1B				(2)				// FDD 1 Busy
#define FDC_ST_D2B				(4)				// FDD 2 Busy
#define FDC_ST_D3B				(8)				// FDD 3 Busy
#define FDC_ST_CB				(0x10)			// FDC Busy
#define FDC_ST_EXM				(0x20)			// Execution Mode
#define FDC_ST_DIO				(0x40)			// Data Input/Output 0:in(PC=>FDU), 1:out(FDU=>PC)
#define FDC_ST_RQM				(0x80)			// Request for Master
// FDC Status Register 0 (ST0)
#define FDC_ST0_IC_INVALID_CMD	(0x80)			// Invalid COmmand issued
#define FDC_ST0_IC_ABNORM_TERM	(0x40)			// Abnormal Temination
#define FDC_ST0_SE				(0x20)			// Seek End
#define FDC_ST0_EC				(0x10)			// Ecuipment Check
#define FDC_ST0_NR				(0x08)			// Not Ready
#define FDC_ST0_HD				(0x04)			// Head Address
#define FDC_ST0_US1				(0x02)			// Unit Select 1
#define FDC_ST0_US0				(0x01)			// Unit Select 0
#define FDC_ST0_US_MASK			(0x03)
// FDC Status Register 1 (ST1)
#define FDC_ST1_EN				(0x80)			// End of Cylinder
#define FDC_ST1_DE				(0x20)			// Data Error
#define FDC_ST1_OR				(0x10)			// Overrun
#define FDC_ST1_ND				(0x04)			// No Data
#define FDC_ST1_NW				(0x02)			// Not Writeable
#define FDC_ST1_MA				(0x01)			// Missing Address Mark
// FDC Status Register 2 (ST2)
#define FDC_ST2_CM				(0x40)			// Control Mark
#define FDC_ST2_DD				(0x20)			// Data Error in Data Field
#define FDC_ST2_WC				(0x10)			// Wrong Cylinder
#define FDC_ST2_SH				(0x08)			// Scan Equal Hit
#define FDC_ST2_SN				(0x04)			// Scan Not Satisfied
#define FDC_ST2_BC				(0x02)			// Bad Cylinder
#define FDC_ST2_MD				(0x01)			// Missing Address Mark in Data Field
// FDC Status Register 3 (ST3)
#define FDC_ST3_FT				(0x80)			// Fault
#define FDC_ST3_WP				(0x40)			// Write Protected
#define FDC_ST3_RY				(0x20)			// Ready
#define FDC_ST3_T0				(0x10)			// Track 0
#define FDC_ST3_TS				(0x08)			// Two Side
#define FDC_ST3_HD				(0x08)			// Head Address
#define FDC_ST3_US1				(0x02)			// Unit Select 1
#define FDC_ST3_US0				(0x01)			// Unit Select 0
#define FDC_ST3_US_MASK			(0x03)
// FDC Command Sequence
#define	FDC_PHASE_IDOL			(0x00)
#define FDC_PHASE_CMD			(0x01)
#define FDC_PHASE_EXE			(0x02)
#define FDC_PHASE_RESULT		(0x04)
#define FDC_PHASE_EXE_SCAN		(0x22)
#define FDC_PHASE_EXE_RD		(0x82)
#define FDC_PHASE_EXE_WD		(0x42)
// FDC Command Code
#define FDC_CMD_RD_DIAG			(0x02)			// Read Diagnostic
#define FDC_CMD_SPECIFY			(0x03)			// Specify
#define FDC_CMD_SNS_DRV_STS		(0x04)			// Sense Drive Status
#define FDC_CMD_WRITE_DAT		(0x05)			// Write Data
#define FDC_CMD_READ_DAT		(0x06)			// Read Data
#define FDC_CMD_RECALIB			(0x07)			// Recalibrate
#define FDC_CMD_SNS_INT_STS		(0x08)			// Sense Interrupt Status
#define FDC_CMD_WRITE_DEL_DAT	(0x09)			// Write Deleted Data
#define FDC_CMD_READ_ID			(0x0A)			// Read ID
#define FDC_CMD_READ_DEL_DAT	(0x0C)			// Read Deleted Data
#define FDC_CMD_WRITE_ID		(0x0D)			// Write ID
#define FDC_CMD_SEEK			(0x0F)			// Seek
//
#define FDC_MAX_UNIT_CNT		(4)
#define FDC_CLK_BY_CLK			(0)
#define FDC_CLK_BY_TC			(1)
#ifdef _MSC_VER
#pragma endregion
#else
// #endregion
#endif
enum FDUDRVBUSY { DRVBSY_IDOL = 0, DRVBSY_READ, DRVBSY_WRITE };
/**--------------------------------------------------------------------------
 **		PROTOTYPEs
 **------------------------------------------------------------------------*/
typedef BOOL (*EXECALLBACK)(DWORD T, LPVOID arg);
struct FDU765A_tag;
typedef struct FDU765A_tag FDU765A, *LPFDU765A;
struct FDUMEMORY_tag;
typedef struct FDUMEMORY_tag FDUMEM, *LPFDUMEM;
struct FDUIOCACHE_tag;
typedef struct FDUIOCACHE_tag FDUIOCACHE, *LPFDUIOCACHE;
struct FDUIOPORT_tag;
typedef struct FDUIOPORT_tag FDUIOPORT, *LPFDUIOPORT;
struct FDU_tag;
typedef struct FDU_tag FDU, *LPFDU;
struct FDUDRIVES_tag;
typedef struct FDUDRIVES_tag FDUDRIVES, *LPFDUDRIVES;
struct FDUDRIVE_tag;
typedef struct FDUDRIVE_tag FDUDRIVE, *LPFDUDRIVE;
struct SECTORID_tag;
typedef struct SECTORID_tag SECTORID, *LPSECTORID;
struct FLOPPYDISK_tag;
typedef struct FLOPPYDISK_tag FLOPPYDISK, *LPFLOPPYDISK;
struct FDUCOMMAND_tag;
typedef struct FDUCOMMAND_tag FDUCOMMAND, *LPFDUCOMMAND;
struct FDURESULT_tag;
typedef struct FDURESULT_tag FDURESULT, *LPFDURESULT;
struct SESSION_tag;
typedef struct SESSION_tag SESSION, *LPSESSION;
typedef void (*LPFNCMEDIAFREE)(LPFLOPPYDISK);
 /**--------------------------------------------------------------------------
 **		STRUCTUREs
 **------------------------------------------------------------------------*/
struct FDUMEMORY_tag {			// Memory map
	BYTE	_rom[8 * 1024];		// ROM
	BYTE	_ram[16 * 1024];	// RAM
	void	(*init)(void);
	void	(*clear)(LPFDUMEM);
	BYTE	(__cdecl* read)(LPFDUMEM, USHORT Adr, DWORD* W);
	void	(__cdecl* write)(LPFDUMEM, USHORT Adr, BYTE v, DWORD* W);
};

struct FDUIOCACHE_tag {
	BYTE	_pio_A_pc;
	BYTE	_pio_B_pc;
	BYTE	_pio_C_pc;
	BYTE	_pio_M_pc;
	BYTE	_pio_A_fdu;
	BYTE	_pio_B_fdu;
	BYTE	_pio_C_fdu;
	BYTE	_pio_M_fdu;
};

struct FDUIOPORT_tag {
	FDUIOCACHE	cache;
	void	(*init)(void);
	void	(*clear)(LPFDU);
	BYTE	(__cdecl* read)(LPFDU, USHORT port);
	void	(__cdecl* write)(LPFDU, USHORT port, BYTE v);
};

PACK(struct SECTORID_tag {
	BYTE	C;					// cylinder
	BYTE	H;					// head side
	BYTE	R;					// sector
	BYTE	N;					// bytes/sector
});
typedef struct SECTORID_tag	SECTORID, *LPSECTORID;

PACK(struct FDUCOMMAND_tag {
	SECTORID	id;
	BYTE		EOT;			// end of track
	BYTE		GPL;			// gap length
	BYTE		DTL;			// data length
	BYTE		STP;

	BYTE		MT;				// Multi-track
	BYTE		MF;				// FM or MFM Mode
	BYTE		SK;				// Skip
	BYTE		HD;				// Head
	BYTE		US;				// Unit Select
	BYTE		SC;				// sectors/track
	BYTE		D;				// filler byte
	BYTE		ND;				// Non-DMA
	BYTE		SRT;			// step rate time
	BYTE		HUT;			// head unload time
	BYTE		HLT;			// head load time
	BYTE		NCN;			// new cylinder no

	BYTE	command_code;
	BYTE	commands[16];
	int		count;

	void	(*clear)(LPFDUCOMMAND);
	BYTE	(*push0)(LPFDUCOMMAND, BYTE);
	int		(*push)(LPFDUCOMMAND, BYTE);
	BOOL	(*load_cmn)(LPFDUCOMMAND);
	BOOL	(*load_std)(LPFDUCOMMAND);
	BOOL	(*load_wid)(LPFDUCOMMAND);
	BOOL	(*load_spec)(LPFDUCOMMAND);
	BOOL	(*load_seek)(LPFDUCOMMAND);
});

struct FDURESULT_tag {
	SECTORID	id;
	BYTE		ST0;
	BYTE		ST1;
	BYTE		ST2;

	BYTE	results[16];
	LPBYTE	ptr;
	int		count;

	void	(*clear)(LPFDURESULT);
	int		(*push0)(LPFDURESULT, BYTE);
	int		(*push)(LPFDURESULT, BYTE);
	int		(*push_std)(LPFDURESULT);
	BYTE	(*pop)(LPFDURESULT);
	void	(*let_std)(LPFDURESULT, LPFDUCOMMAND);
	void	(*let_id)(LPFDURESULT, LPSECTORID);

};

struct SESSION_tag {
	LPFDU765A	fdc;
	const char* name;
	int		multi_trk_count;
	int		data_count;
	void	(*command)(LPSESSION, BYTE);
	void	(*write_data)(LPSESSION, BYTE);
	BYTE	(*read_data)(LPSESSION);
	void	(*clk)(LPSESSION, DWORD, BYTE);
};

struct FDU765A_tag {
	LPFDU		owner;
	BYTE		status;
	BYTE		phase;
	FDUCOMMAND	cmd;
	FDURESULT	result;
	SESSION		_session;
	LPSESSION	current;

	//LPBYTE	data_ptr;
	int		rw_data_cnt;

	int		step_rate_time;		// SRT: 1..16ms (1ms unit)
	int		head_unload_time;	// HUT: 16..240ms (16ms unit)
	int		head_load_time;		// HLT: 16..240ms (16ms unit)
	BOOL	non_dma_mode;		// ND: Non-DMA mode

	void	(*init)(LPFDU765A, LPFDU);
	void	(*reset)(LPFDU765A);
	BYTE	(*read_status_reg)(LPFDU765A);
	BOOL	(*rd_ready)(LPFDU765A);		// read data ready
	void	(*write)(LPFDU765A, BYTE data);
	BYTE	(*read)(LPFDU765A);
	void	(*TC)(LPFDU765A);
	void	(*enter_idol_phase)(LPFDU765A);
	void	(*enter_cmd_phase)(LPFDU765A);
	void	(*enter_result_phase)(LPFDU765A);
	void	(*enter_exe_phase)(LPFDU765A, int unit);
};

PACK(struct D88HEADER_tag {
	char	title[17];
	BYTE	reserved[9];
	BYTE	readonly;			// Write Protect: 0x00(No), 0x10(Yes)
	BYTE	disktype;			// 0x00:2D, 0x10:2DD, 0x20: 2HD, 0x30:1D, 0x40:1DD
	BYTE	disksize[4];		// image size
	BYTE	trackptr[164*4];	// offset to track data
});
typedef struct D88HEADER_tag D88HEADER, *LPD88HEADER;

PACK(struct D88SECTOR_tag {
	SECTORID	id;				// sector ID
	BYTE	sectors[2];			// amount of sectors
	BYTE	density;			// 0x00:double, 0x40:single, 0x01:high
	BYTE	deleted;			// 0x00:normal, 0x10:deleted
	BYTE	status;				// 0x00:normal, 0x10:normal(deleted), 0xA0:id crc err, 0xB0:data crc drr, 0xE0:non addr mark, 0xF0:non data mark
	BYTE	reserve[5];
	BYTE	length[2];
});
typedef struct D88SECTOR_tag D88SECTOR, *LPD88SECTOR;

PACK(struct D88SECTOR_EX_tag {
	D88SECTOR	header;
	BYTE		data[1024];
});
typedef struct D88SECTOR_EX_tag D88SECTOR_EX, *LPD88SECTOR_EX;

struct FLOPPYDISK_tag {			// floppy disk media
	LPBYTE	image_raw_data;		// pointer to image raw data
	DWORD	image_length;		// image length
	LPFNCMEDIAFREE	free;		// release image data method
	LPD88HEADER		header;
	const char*		err_msg;
	LPD88SECTOR_EX	(*find_sector)(LPFLOPPYDISK, LPSECTORID);
};

struct FDUDRIVE_tag {					// Disk Drive
	LPFDUDRIVES		owner;
	LPFLOPPYDISK	media;
	BYTE			unit_no;			// Drive No.
	BOOL			missing_unit;		// No Drive Unit
	BOOL			media_inserted;
	BYTE			status;
	BYTE			track;				// current cylinder
	char			state;				// -:N/A, I:IDOL, R:Read, W:Write, S:Seek
	SECTORID		id;					// target id (r/w/s)
	DWORD			seek_interval;

	DWORD			remain_cnt;
	BYTE			data_buff[1024];
	int				data_size;
	int				data_buff_index;
	enum FDUDRVBUSY	busy;
	void	(*reset)(LPFDUDRIVE);
	void	(*eject)(LPFDUDRIVE);
	BOOL	(*insert)(LPFDUDRIVE, LPFLOPPYDISK);
	char	(*get_state)(LPFDUDRIVE);
	void	(*init)(LPFDUDRIVE, BYTE unit_no);
};

struct FDUDRIVES_tag {
	LPFDU		owner;
	FDUDRIVE	drives[4];			// Floppy Disk Drives
	int			drives_cnt;			// amount of Disk Drives
	int			(*seek)(LPFDUDRIVES, int unit, int track);
	void		(*snsdrv)(LPFDUDRIVES, int unit, int head);
	void		(*snsint)(LPFDUDRIVES);
	void		(*reset)(LPFDUDRIVES);
	int			(*read)(LPFDUDRIVES, int unit, LPFDUCOMMAND, int mt_cnt);
	void		(*init)(LPFDUDRIVES, LPFDU);
};

typedef struct PD8255_tag {		// 8255 PPI
	BYTE	reg_mode;
	void	(*outA)(BYTE data);
	void	(*outB)(BYTE data);
	void	(*outC)(BYTE data);
	BYTE	(*inA)(void);
	BYTE	(*inB)(void);
	BYTE	(*inC)(void);
	void	(*outM)(BYTE data);
	BYTE	(*inM)(void);
}	PD8255, *LPPD8255;

typedef struct FDUIF_tag {		// PC<=>FDU I/F (PC-8011/12/33?)
	LPFDU	owner;
	PD8255	intl;				// internal PPI (FDU side)
	PD8255	extl;				// external PPI (PC side)
	char	state1;				// PC  current i/o mode
	char	state2;				// FDU current i/o mode
	void	(*init)(void);
}	FDUIF, *LPFDUIF;

struct FDU_tag {				// PC-8031 Floppy Disk Unit
	Z80CPU		cpu;			// PC-8031 cpu
	FDUMEM		mem;			// PC-8031 memory
	FDUIF		ppi;			// 8255(PC)<=>8255(FDU)
	FDUIOPORT	io;				// i/o port cache
	FDU765A		fdc;			// uPD765A Floppy Disk Controller
	FDUDRIVES	fdd;			// Disk Drives
	BOOL		rom_ready;		// FDU ROM loaded
	BOOL		cpu_sleeping;	// FDU cpu sleeping
	DWORD		cpu_T_limit;	// FDU cpu available ticks

	void		(*set_rom_ready)(LPFDU, BOOL flag);
	DWORD		(*msec_to_T)(LPFDU, double msec);

	BOOL		(*media_init)(LPFDU, LPFLOPPYDISK, LPVOID, DWORD, LPFNCMEDIAFREE);
	void		(*reset)(LPFDU);
	void		(*dispose)(LPFDU);
	void		(*resume)(LPFDU);
	BOOL		(*available)(LPFDU);

	struct N80DEVICE_tag	*owner;
};

#ifdef __cplusplus
}	// extern "C"
#endif
#endif//_PC_8031_H_
