#ifndef _N80TYPES_H_
#define _N80TYPES_H_
#include <stdint.h>
#include <stdbool.h>
/**--------------------------------------------------------------------------
 **    declare types
 **------------------------------------------------------------------------*/
typedef uint8_t         BYTE, *LPBYTE;
typedef const uint8_t   CBYTE, *LPCBYTE;
typedef uint16_t        USHORT, *LPUSHORT;
typedef const uint16_t  CUSHORT, *LPCUSHORT;
typedef uint32_t        DWORD, UINT, ULONG, *LPDWORD;
typedef const uint32_t  CDWORD, *LPCDWORD;
typedef int32_t         LONG, *LPLONG;
typedef int16_t         SHORT, *LPSHORT;
typedef void            *LPVOID;
typedef long long       LONGLONG;
typedef unsigned long long ULONGLONG;
#ifdef __APPLE__
//typedef _Bool            BOOL;
#define BOOL            _Bool
#else
typedef bool            BOOL;
#endif
#ifndef NULL
#define NULL    ((LPVOID) 0)
#endif
#ifdef __linux__
/**--------------------------------------------------------------------------
 **    inline functions
 **------------------------------------------------------------------------*/
#if defined(_RASPBERRY_)
static inline DWORD interlocked_exchange(DWORD volatile *target, DWORD new_value)
{
    ULONG    xcg_value;
    
    __asm__ __volatile__ (
    "        ldr    r0,%1            \n\t"
    "0:        ldrex    r2,[r0]        \n\t"
    "        mov    r3,r2            \n\t"
    "        mov    r2,%2            \n\t"
    "        strex    r1,r2,[r0]    \n\t"
    "        cmp    r1,#0            \n\t"
    "        bne    0b                \n\t"
    "        str    r3,%0            \n\t"
    : "=m"(xcg_value)
    : "m"(target),"r"(new_value)
    : "r0","r1","r2","r3"
    );
    return xcg_value;
}

static inline DWORD interlocked_exchangeadd(DWORD volatile *target, DWORD add_value)
{
    ULONG    xcg_value;
    
    __asm__ __volatile__ (
    "        ldr    r0,%1            \n\t"
    "0:        ldrex    r2,[r0]        \n\t"
    "        mov    r3,r2            \n\t"
    "        add    r2,r2,%2        \n\t"
    "        strex    r1,r2,[r0]    \n\t"
    "        cmp    r1,#0            \n\t"
    "        bne    0b                \n\t"
    "        str    r3,%0            \n\t"
    : "=m"(xcg_value)
    : "m"(target),"r"(add_value)
    : "r0","r1","r2","r3"
    );
    return xcg_value;
}

static inline DWORD interlocked_increment(DWORD volatile *target)
{
    ULONG    inc_value;
    
    __asm__ __volatile__ (
    "        ldr    r0,%1            \n\t"
    "0:        ldrex    r2,[r0]        \n\t"
    "        add    r2,r2,#1        \n\t"
    "        strex    r1,r2,[r0]    \n\t"
    "        cmp    r1,#0            \n\t"
    "        bne    0b                \n\t"
    "        str    r2,%0            \n\t"
    : "=m"(inc_value)
    : "m"(target)
    : "r0","r1","r2"
    );
    return inc_value;
}

static inline DWORD interlocked_decrement(DWORD volatile *target)
{
    ULONG    dec_value;
    
    __asm__ __volatile__ (
    "        ldr    r0,%1            \n\t"
    "0:        ldrex    r2,[r0]        \n\t"
    "        sub    r2,r2,#1        \n\t"
    "        strex    r1,r2,[r0]    \n\t"
    "        cmp    r1,#0            \n\t"
    "        bne    0b                \n\t"
    "        str    r2,%0            \n\t"
    : "=m"(dec_value)
    : "m"(target)
    : "r0","r1","r2"
    );
    return dec_value;
}
/*
#else
static inline DWORD interlocked_exchange(DWORD volatile *target, DWORD new_value)
{
    ULONG    xcg_value;
    
    __asm__ __volatile__ (
    "        ldr    r0,%1            \n\t"
    "        swp    r1,%2,[r0]        \n\t"
    "        str    r1,%0            \n\t"
    : "=m"(xcg_value)
    : "m"(target),"r"(new_value)
    : "r0","r1"
    );
    return xcg_value;
}

static inline void spin_lock(DWORD volatile *target)
{
    __asm__ __volatile__ (
    "        ldr    r0,%0            \n\t"
    "        mov    r2,#1            \n\t"
    "0:        swp    r1,r2,[r0]        \n\t"
    "        cmp    r1,#0            \n\t"
    "        bne    0b                \n\t"
    :
    : "m"(target)
    : "r0","r1","r2"
    );
}

static inline void spin_unlock(DWORD volatile *target)
{
    __asm__ __volatile__ (
    "        ldr    r0,%0            \n\t"
    "        eor    r1,r1,r1        \n\t"
    "        swp    r1,r1,[r0]        \n\t"
    :
    : "m"(target)
    : "r0","r1"
    );
}*/
#elif defined(__x86_64)
static inline int interlocked_exchangeadd(volatile DWORD *target, int value) {
    __asm__ __volatile__ (
        "lock xadd %0,(%1);"
        : "=a"(value)
        : "r"(target), "a"(value)
        : "memory"
    );
    return value;
}
static inline int interlocked_exchange(volatile DWORD *target, int value) {
    __asm__ __volatile__ (
        "lock xchg %0,(%1);"
        : "=a"(value)
        : "r"(target), "a"(value)
        : "memory"
    );
    return value;
}
#endif
#endif
#endif
