//
//  n80config.h
//  N80X
//
//  Copyright (C) 1993-2024 Hiroshi Kanemoto.
//    Released under the MIT license
//    http://opensource.org/licenses/mit-license.php
//
//  Created by hiroshi on 2024/05/15.
//
#ifndef __N80CONFIG__
#define __N80CONFIG__
#import <GameController/GCKeyCodes.h>
#import "n80em.h"
#include <sys/ioctl.h>
#include "n80.h"
#include "PC-8001.h"
//---------------------------------------------------------------------------
//	declare structures
//---------------------------------------------------------------------------
typedef void* LPVOID;
typedef struct tagITEM_ {
	int     type;
	char*   name;
	char*   hint;
	int		value;
	int     (*controller)(struct tagITEM_*, int, LPVOID);
    LPVOID  arg;
}	ITEM, *LPITEM;
typedef struct tagTEXTIN_ARG {
    char*  org;
    char*  tmp;
    LPVOID  aux1;
    LPVOID  aux2;
}   TEXTIN_ARG, *LPTEXTIN_ARG;
typedef struct tagKEYEVT {
	GCKeyCode  key;
	BYTE	code;	// PC-8001 Key Code
	bool    SHIFT;
	bool    ALT;
	bool    CTL;
	char    ascii;
}   KEYEVTG, * LPKEYEVT;
/**--------------------------------------------------------------------------
 **	declare macro
 **------------------------------------------------------------------------*/
#define TYPE_TOGGLE     (0)     // toggle control
#define TYPE_TEXTIN     (1)     // text-in control
#define TYPE_UPDOWN     (2)     // value up-down control
#define TYPE_INFO       (3)     // readonly information control
#define TYPE_TERMINATOR (-1)    // terminator dummy control
#define VERB_INIT       (0)     // INIT control
#define VERB_EVENT      (1)     // Key Event handle
#define VERB_PRINT      (2)     // Print control
#define VERB_PRINT_SEL  (3)     // Print control (selected status)
#define VERB_TOGGLE_SET (4)     // on toggled set value
#define VERB_UPDOWN_SET (4)     // on up-down set value
#define VERB_TEXTIN_CH  (4)     // on key char
#define VERB_TEXTIN_SET (5)     // on text-in set value
//-----------------------------------------------------------------------------
//    Emulator Class
//-----------------------------------------------------------------------------
@class n80config_data;
@interface n80config : NSObject {
	@private
	n80em			*n80em;
	n80config_data	*data;
}

@property (nonatomic, assign, readonly, getter=getParam) n80config_data const *param;

-(id)initWithN80:(n80em*)n80em config:(n80config_data*)config;
-(void)enter_config;
-(void)leave_config;
-(void)OnKeyDown:(BYTE)key code:(GCKeyCode)c;
-(bool)OnKeyUp:(BYTE)key code:(GCKeyCode)c;
-(void)drawitem:(LPVOID)_item line:(int)line selected:(bool)selected;
-(void)reflect_menu_state;

@end

@interface n80config_data : NSObject {
	@public
	char	slot[3][64];
}
@property (assign) BOOL interlace;
@property (assign) BOOL hires_crt;
@property (assign) BOOL tenkeyless;
@property (assign) BOOL chiqlappe;
@property (assign) BOOL nowait;

-(id)init;
-(void)set_char:(char)c index:(int)index slot:(int)slot;
-(char)get_char:(int)index slot:(int)slot;
-(int)get_size:(int)slot;
@end

#endif//__N80CONFIG__
