//
//  n80config.m
//
//  Copyright (C) 1993-2024 Hiroshi Kanemoto.
//    Released under the MIT license
//    http://opensource.org/licenses/mit-license.php
//
//    Created by hiroshi on 2024/05/15.
//
#import <Foundation/Foundation.h>
#import <N80X-Swift.h>
#include <ctype.h>
#include <string.h>
#import "n80em.h"
#import "n80config.h"
#import "n80keyboard.h"
#import "n80graph.h"
//-----------------------------------------------------------------------------
//  local function declaration
//-----------------------------------------------------------------------------
static int _controller_crt(LPITEM, int, LPVOID);
static int _controller_use_newpcg(LPITEM, int, LPVOID);
static int _controller_tenkeyless(LPITEM, int, LPVOID);
static int _controller_interrace(LPITEM, int, LPVOID);
static int _controller_nowait(LPITEM, int, LPVOID);
static int _controller_diskrom(LPITEM, int, LPVOID);
static int _controller_n80rom(LPITEM, int, LPVOID);
static int _controller_4throm(LPITEM, int, LPVOID);
static int _controller_fdurom(LPITEM, int, LPVOID);
//-----------------------------------------------------------------------------
//	local variables
//-----------------------------------------------------------------------------
static char tmp_n80_rom[64];
static char tmp_4th_rom[64];
static char tmp_fdu_rom[64];
static DWORD tmp_save_screen_mode;
static BYTE  tmp_save_text_visible;
static BOOL  tmp_save_use_new_pcg;
static bool  tmp_tenkeyless = true;
static bool  tmp_z80_nowait = false;
static TEXTIN_ARG args[] = {
    {NULL, tmp_n80_rom, "PC-8001", ".ROM"},
    {NULL, tmp_4th_rom, "", ".ROM"},
    {NULL, tmp_fdu_rom, "", ".ROM"},
};
static ITEM items[] = {
	{TYPE_TOGGLE, "HiRes-CRT (24kHz/400Lines)............ [ ]", "select CRT from 24kHz or 15kHz(Normal/200Lines)", 0, _controller_crt, NULL},
	{TYPE_TOGGLE, "TEN-Keyless .......................... [ ]", "assign Ten-Key (2/4/6/8) to cursor key for Tenkey-less keyboard", 0, _controller_tenkeyless, NULL},
	{TYPE_TOGGLE, "Interlace-like effect ................ [ ]", "display like interlace monitor", 0, _controller_interrace, NULL},
    {TYPE_TOGGLE, "No-wait .............................. [ ]", "ignore memory wait, M1 wait only", 0, _controller_nowait, NULL},
    {TYPE_TOGGLE, "New PCG (chiqlappe) .................. [ ]", "use new pcg (chiqlappeｻﾝ) device", 0, _controller_use_newpcg, NULL},
    {TYPE_TEXTIN, "N80 rom file name .................... [                        ]", "N-BASIC ROM default file name (required)", 24, _controller_n80rom, &args[0]},
    {TYPE_TEXTIN, "4th rom file name .................... [                        ]", "4TH ROM default file name (option)", 24, _controller_4throm, &args[1]},
    {TYPE_TEXTIN, "Disk Unit rom file name .............. [                        ]", "Disk ROM default file name (option)", 24, _controller_fdurom, &args[2]},
    {TYPE_INFO,   "disk rom status ...................... [   ]", "disk rom is available or not", 0, _controller_diskrom, NULL},
    {TYPE_TERMINATOR}
};
// #region Windows Virtual Key<=>ASCII Code
static char to_ascii(LPKEYEVT kv) {
    if (kv->key == GCKeyCodeOne)        /* 1 */	return kv->SHIFT ? 0x21 : 0x31;
    else if (kv->key == GCKeyCodeTwo)   /* 2 */	return kv->SHIFT ? 0x22 : 0x32;
    else if (kv->key == GCKeyCodeThree)	/* 3 */	return kv->SHIFT ? 0x23 : 0x33;
    else if (kv->key == GCKeyCodeFour)  /* 4 */	return kv->SHIFT ? 0x24 : 0x34;
    else if (kv->key == GCKeyCodeFive)  /* 5 */	return kv->SHIFT ? 0x24 : 0x35;
    else if (kv->key == GCKeyCodeSix)   /* 6 */	return kv->SHIFT ? 0x26 : 0x36;
    else if (kv->key == GCKeyCodeSeven)	/* 7 */	return kv->SHIFT ? 0x27 : 0x37;
    else if (kv->key == GCKeyCodeEight)	/* 8 */	return kv->SHIFT ? 0x28 : 0x38;
    else if (kv->key == GCKeyCodeNine)  /* 9 */	return kv->SHIFT ? 0x29 : 0x39;
    else if (kv->key == GCKeyCodeZero)  /* 0 */	return kv->SHIFT ? 0x30 : 0x30;

    else if (kv->key == GCKeyCodeKeyQ)	/* Q */	return kv->SHIFT ? 0x51 : 0x71;
    else if (kv->key == GCKeyCodeKeyW)	/* W */	return kv->SHIFT ? 0x57 : 0x77;
    else if (kv->key == GCKeyCodeKeyE)	/* E */	return kv->SHIFT ? 0x45 : 0x65;
    else if (kv->key == GCKeyCodeKeyR)	/* R */	return kv->SHIFT ? 0x52 : 0x72;
    else if (kv->key == GCKeyCodeKeyT)	/* T */	return kv->SHIFT ? 0x54 : 0x74;
    else if (kv->key == GCKeyCodeKeyY)	/* Y */	return kv->SHIFT ? 0x59 : 0x79;
    else if (kv->key == GCKeyCodeKeyU)	/* U */	return kv->SHIFT ? 0x55 : 0x75;
    else if (kv->key == GCKeyCodeKeyI)	/* I */	return kv->SHIFT ? 0x49 : 0x69;
    else if (kv->key == GCKeyCodeKeyO)	/* O */	return kv->SHIFT ? 0x4F : 0x6F;
    else if (kv->key == GCKeyCodeKeyP)	/* P */	return kv->SHIFT ? 0x50 : 0x70;

    else if (kv->key == GCKeyCodeKeyA)	/* A */	return kv->SHIFT ? 0x41 : 0x61;
    else if (kv->key == GCKeyCodeKeyS)	/* S */	return kv->SHIFT ? 0x53 : 0x73;
    else if (kv->key == GCKeyCodeKeyD)	/* D */	return kv->SHIFT ? 0x44 : 0x64;
    else if (kv->key == GCKeyCodeKeyF)	/* F */	return kv->SHIFT ? 0x46 : 0x66;
    else if (kv->key == GCKeyCodeKeyG)	/* G */	return kv->SHIFT ? 0x47 : 0x67;
    else if (kv->key == GCKeyCodeKeyF)	/* H */	return kv->SHIFT ? 0x48 : 0x68;
    else if (kv->key == GCKeyCodeKeyJ)	/* J */	return kv->SHIFT ? 0x4A : 0x6A;
    else if (kv->key == GCKeyCodeKeyK)	/* K */	return kv->SHIFT ? 0x4B : 0x6B;
    else if (kv->key == GCKeyCodeKeyL)	/* L */	return kv->SHIFT ? 0x4C : 0x6C;

    else if (kv->key == GCKeyCodeKeyZ)	/* Z */	return kv->SHIFT ? 0x5A : 0x7A;
    else if (kv->key == GCKeyCodeKeyX)	/* X */	return kv->SHIFT ? 0x58 : 0x78;
    else if (kv->key == GCKeyCodeKeyC)	/* C */	return kv->SHIFT ? 0x43 : 0x63;
    else if (kv->key == GCKeyCodeKeyV)	/* V */	return kv->SHIFT ? 0x56 : 0x76;
    else if (kv->key == GCKeyCodeKeyB)	/* B */	return kv->SHIFT ? 0x42 : 0x62;
    else if (kv->key == GCKeyCodeKeyN)	/* N */	return kv->SHIFT ? 0x4E : 0x6E;
    else if (kv->key == GCKeyCodeKeyM)	/* M */	return kv->SHIFT ? 0x4D : 0x6D;

    else if (kv->key == GCKeyCodeEscape)               /*ESC*/ return 0x1B;
    else if (kv->key == GCKeyCodeDeleteOrBackspace)    /* BS*/ return 0x08;
    else if (kv->key == GCKeyCodeDeleteOrBackspace)    /*DEL*/ return 0x08;
    else if (kv->key == GCKeyCodeTab)                  /*TAB*/ return 0x09;
    else if (kv->key == GCKeyCodeReturnOrEnter)        /*RET*/ return 0x0D;

    else if (kv->key == GCKeyCodeKeypad0)	/* 0 */     return 0x30;
    else if (kv->key == GCKeyCodeKeypad1)	/* 1 */     return 0x31;
    else if (kv->key == GCKeyCodeKeypad2)	/* 2 */     return 0x32;
    else if (kv->key == GCKeyCodeKeypad3)	/* 3 */     return 0x33;
    else if (kv->key == GCKeyCodeKeypad4)	/* 4 */     return 0x34;
    else if (kv->key == GCKeyCodeKeypad5)	/* 5 */     return 0x35;
    else if (kv->key == GCKeyCodeKeypad6)	/* 6 */     return 0x36;
    else if (kv->key == GCKeyCodeKeypad7)	/* 7 */     return 0x37;
    else if (kv->key == GCKeyCodeKeypad8)	/* 8 */     return 0x38;
    else if (kv->key == GCKeyCodeKeypad9)	/* 9 */     return 0x39;
    else if (kv->key == GCKeyCodeKeypadAsterisk)        /* * */ return 0x2a;
    else if (kv->key == GCKeyCodeKeypadPlus)            /* + */ return 0x2b;
    else if (kv->key == GCKeyCodeKeypadEqualSign)       /* = */ return 0x3d;
    else if (kv->key == GCKeyCodeKeypadPeriod)          /* . */ return 0x2e;

    else if (kv->key == GCKeyCodeHyphen)	            /* = -   */	return kv->SHIFT ? 0x3D : 0x2D;
    else if (kv->key == GCKeyCodeGraveAccentAndTilde)   /* ^ ~   */	return kv->SHIFT ? 0x7E : 0x5E;
    else if (kv->key == GCKeyCodeOpenBracket)           /* @ `   */	return kv->SHIFT ? 0x60 : 0x40;
    else if (kv->key == GCKeyCodeCloseBracket)	        /* [ {   */	return kv->SHIFT ? 0x7B : 0x5B;
    else if (kv->key == GCKeyCodeSemicolon)             /* + ;   */	return kv->SHIFT ? 0x2B : 0x3B;
    else if (kv->key == GCKeyCodeQuote)                 /* * :   */	return kv->SHIFT ? 0x2A : 0x3A;
    else if (kv->key == GCKeyCodeCloseBracket)          /* ] }   */	return kv->SHIFT ? 0x7D : 0x5D;
    else if (kv->key == GCKeyCodeComma)                 /* < ,   */	return kv->SHIFT ? 0x3C : 0x2C;
    else if (kv->key == GCKeyCodePeriod)                /* > .   */	return kv->SHIFT ? 0x3E : 0x2E;
    else if (kv->key == GCKeyCodeSlash)                 /* / ?   */	return kv->SHIFT ? 0x3F : 0x2F;
    else if (kv->key == GCKeyCodeInternational1)        /* \ _   */	return kv->SHIFT ? 0x5F : 0x5C;
    else if (kv->key == GCKeyCodeInternational3)        /* \ |   */	return kv->SHIFT ? 0x7C : 0x5C;
    else    return '\0';
}
// #endregion
static int  item_cnt;
static int  cur_idx;
static char save_cmt_name[12];
static char save_cmt_dir[12];
static KEYEVTG keyevt;
static char m_cui_text[25][80];
static char m_cui_attr[25][80];
//-----------------------------------------------------------------------------
//	functions
//-----------------------------------------------------------------------------
static void utf8_to_sjis(const char *target, char *buff, int siz) {
    NSString *utf8 = [[NSString alloc] initWithUTF8String:(void *)target];
    NSData *sjis = [utf8 dataUsingEncoding:NSShiftJISStringEncoding];
    memset(buff, 0, siz);
    if (sjis.length > 0) {
        memcpy(buff, sjis.bytes, sjis.length);
    }
}
static bool is_alpha(LPKEYEVT kv) {
    if (kv->ascii >= 'a' && kv->ascii <= 'z')
        return true;
    if (kv->ascii >= 'A' && kv->ascii <= 'Z')
        return true;
    return false;
}
static bool is_numeric(LPKEYEVT kv) {
    return (kv->ascii >= '0' && kv->ascii <= '9');
}
// #region toggle control methods
static int _toggle_event(LPITEM self, LPKEYEVT key) {
    if (key->key == GCKeyCodeSpacebar) { // [SP] Key pressed
        self->value = self->value == 1 ? 0 : 1;
        self->controller(self, VERB_TOGGLE_SET, NULL);
        return 1;
    }
    else
        return 0;
}
static void _toggle_print(LPITEM self, char* buff) {
    memset(buff, 32, 78);
    strcpy(buff, self->name);
    if (self->value == 1)
        buff[40] = 'X';
}
// #endregion
// #region text input control methods
static void CharLower(char *buff) {
    size_t l = strlen(buff);
    for (int i = 0; i < l; ++i)
        buff[i] = (char)tolower(buff[i]);
}
static void _textin_filename_init(LPITEM self, int maxlen) {
    LPTEXTIN_ARG arg = (LPTEXTIN_ARG)self->arg;
    memset(arg->tmp, 0, maxlen);
    strcpy(arg->tmp, arg->org);
}
static int _textin_event(LPITEM self, LPKEYEVT kv) {
    LPTEXTIN_ARG arg = (LPTEXTIN_ARG)self->arg;
    char* buff = arg->tmp;
    int maxlen = self->value - 1;
    size_t len;
    if (kv->key == GCKeyCodeTab || kv->key == GCKeyCodeUpArrow || kv->key == GCKeyCodeDownArrow) // [TAB] | [ESC] Key pressed
        return self->controller(self, VERB_TEXTIN_SET, kv);
    len = strlen(buff);
    if (len > maxlen)
        return 0;
    if (kv->key == GCKeyCodeDeleteOrBackspace && len > 0) { // [BS] Key pressed
        buff[len - 1] = '\0';
        return 1;
    }
    else if (len < maxlen && self->controller(self, VERB_TEXTIN_CH, kv) == 1) {
        buff[len] = kv->ascii;
        buff[len + 1] = '\0';
    }
    return 1;
}
static void _textin_print(LPITEM self, char* vram, bool selected) {
    LPTEXTIN_ARG arg = (LPTEXTIN_ARG)self->arg;
    int maxlen = self->value - 1;
    char* src = arg->tmp;
    size_t l = strlen(src);
    memset(vram, 0x20, 78);
    strcpy(vram, self->name);          // item title format
    for (int i = 0; i < maxlen; ++i) {  // item value (string)
        char ch = *src++;
        if (ch == '\0')
            break;
        vram[40 + i] = ch;
    }
    if (selected) { // show guide cursor at eot when selected
        if (l < maxlen)
            vram[40 + l] = 0x1D;
    }
}
static int _textin_filename_ch(LPITEM self, LPKEYEVT kv) {
    LPTEXTIN_ARG arg = (LPTEXTIN_ARG)self->arg;
    if (is_alpha(kv) || is_numeric(kv))
        return 1;
    if (kv->ascii == '_' || kv->ascii == '.')
        return 1;
    if (kv->ascii == '-' && strlen(arg->tmp) > 0)
        return 1;
    return 0;
}
static int _textin_filename_set(LPITEM self) {
    LPTEXTIN_ARG arg = (LPTEXTIN_ARG)self->arg;
    memset(arg->org, 0, self->value);
    char* fn = strlen(arg->tmp) > 0 ? arg->tmp : (char*)arg->aux1;
    if (strlen(fn) == 0)
        return 0;
    strcat(arg->org, fn);
    //strcat(arg->org, (char*)arg->aux2);
    return 0;
}
// #endregion
// #region value control methods
static int _updown_event(LPITEM self, LPKEYEVT key, int minvalue, int maxvalue) {
    if (key->key == GCKeyCodeLeftArrow) {
        if (self->value > minvalue) {
            --self->value;
            self->controller(self, VERB_UPDOWN_SET, NULL);
            return 1;
        }
    }
    else if (key->key == GCKeyCodeRightArrow) {
        if (self->value < maxvalue) {
            ++self->value;
            self->controller(self, VERB_UPDOWN_SET, NULL);
            return 1;
        }
    }
    return 0;
}
static void _updown_print(LPITEM self, char* buff, int index, int maxlen) {
    char tmp[64];
    memset(buff, 32, 78);
    strcpy(buff, self->name);
    memset(tmp, 0, 64);
    sprintf(tmp, "%d", self->value);
    char* ptr = &buff[index];
    size_t len = strlen(tmp);
    if (len < maxlen) {
        for (int i = 0; i < (maxlen - len); ++i)
            *ptr++ = 32;
    }
    strcpy(ptr, tmp);
    buff[index + maxlen] = ']';
}
// #endregion
// #region config items
static int _controller_crt(LPITEM self, int verb, LPVOID param) {
    switch (verb) {
    case VERB_INIT:
        self->value = n80device.vrtc.crt_hires_mode == _CRT_HIRES ? 1 : 0;
        break;
    case VERB_EVENT:
        return _toggle_event(self, (LPKEYEVT)param);
    case VERB_PRINT:
    case VERB_PRINT_SEL:
        _toggle_print(self, (char*)param);
        break;
    case VERB_TOGGLE_SET:
        n80device.vrtc.crt_hires_mode = self->value == 1 ? _CRT_HIRES : _CRT_NORMAL;
        break;
    }
    return 0;
}
static int _controller_use_newpcg(LPITEM self, int verb, LPVOID param) {
    switch (verb) {
    case VERB_INIT:
        self->value = tmp_save_use_new_pcg ? 1 : 0;
        break;
    case VERB_EVENT:
        return _toggle_event(self, (LPKEYEVT)param);
    case VERB_PRINT:
    case VERB_PRINT_SEL:
        _toggle_print(self, (char*)param);
        break;
    case VERB_TOGGLE_SET:
        tmp_save_use_new_pcg = self->value == 1;
        break;
    }
    return 0;
}
static int _controller_tenkeyless(LPITEM self, int verb, LPVOID param) {
    switch (verb) {
    case VERB_INIT:
        self->value = tmp_tenkeyless ? 1 : 0;
        break;
    case VERB_EVENT:
        return _toggle_event(self, (LPKEYEVT)param);
    case VERB_PRINT:
    case VERB_PRINT_SEL:
        _toggle_print(self, (char*)param);
        break;
    case VERB_TOGGLE_SET:
        tmp_tenkeyless = self->value == 1;
        break;
    }
    return 0;
}
static int _controller_interrace(LPITEM self, int verb, LPVOID param) {
    switch (verb) {
    case VERB_INIT:
        self->value = n80device.param.interlace ? 1 : 0;
        break;
    case VERB_EVENT:
        return _toggle_event(self, (LPKEYEVT)param);
    case VERB_PRINT:
    case VERB_PRINT_SEL:
        _toggle_print(self, (char*)param);
        break;
    case VERB_TOGGLE_SET:
        n80device.param.interlace = self->value == 1;
        n80device.blt.chg_palette(n80device.param.interlace);
        break;
    }
    return 0;
}
static int _controller_nowait(LPITEM self, int verb, LPVOID param) {
    switch (verb) {
    case VERB_INIT:
        self->value = tmp_z80_nowait ? 1 : 0;
        break;
    case VERB_EVENT:
        return _toggle_event(self, (LPKEYEVT)param);
    case VERB_PRINT:
    case VERB_PRINT_SEL:
        _toggle_print(self, (char*)param);
        break;
    case VERB_TOGGLE_SET:
        tmp_z80_nowait = self->value == 1;
        break;
    }
    return 0;
}
static int _controller_diskrom(LPITEM self, int verb, LPVOID param) {
    switch (verb) {
    case VERB_INIT:
        self->value = n80device.fdu.rom_ready ? 1 : 0;
        break;
    case VERB_EVENT:
        return 0;
    case VERB_PRINT:
    case VERB_PRINT_SEL: {
        char *buff = (char*)param;
        memset(buff, 32, 78);
        strcpy(buff, self->name);
        if (self->value == 1) {
            buff[40] = 'R';
            buff[41] = 'D';
            buff[42] = 'Y';
        } else {
            buff[40] = 'N';
            buff[41] = '/';
            buff[42] = 'A';
        }
        break;
    }
    case VERB_TOGGLE_SET:
        break;
    }
    return 0;
}
static int _controller_n80rom(LPITEM self, int verb, LPVOID param) {
    switch (verb) {
    case VERB_INIT:     //erace file extension
        _textin_filename_init(self, self->value);
        break;
    case VERB_EVENT:
        return _textin_event(self, (LPKEYEVT)param);
    case VERB_PRINT:
    case VERB_PRINT_SEL:
        _textin_print(self, (char*)param, verb == VERB_PRINT_SEL);
        break;
    case VERB_TEXTIN_CH:
        return _textin_filename_ch(self, (LPKEYEVT)param);
    case VERB_TEXTIN_SET:
        return _textin_filename_set(self);
    }
    return 0;
}
static int _controller_4throm(LPITEM self, int verb, LPVOID param) {
    switch (verb) {
    case VERB_INIT:     //erace file extension
        _textin_filename_init(self, self->value);
        break;
    case VERB_EVENT:
        return _textin_event(self, (LPKEYEVT)param);
    case VERB_PRINT:
    case VERB_PRINT_SEL:
        _textin_print(self, (char*)param, verb == VERB_PRINT_SEL);
        break;
    case VERB_TEXTIN_CH:
        return _textin_filename_ch(self, (LPKEYEVT)param);
    case VERB_TEXTIN_SET:
        return _textin_filename_set(self);
    }
    return 0;
}
static int _controller_fdurom(LPITEM self, int verb, LPVOID param) {
    switch (verb) {
    case VERB_INIT:     //erace file extension
        _textin_filename_init(self, self->value);
        break;
    case VERB_EVENT:
        return _textin_event(self, (LPKEYEVT)param);
    case VERB_PRINT:
    case VERB_PRINT_SEL:
        _textin_print(self, (char*)param, verb == VERB_PRINT_SEL);
        break;
    case VERB_TEXTIN_CH:
        return _textin_filename_ch(self, (LPKEYEVT)param);
    case VERB_TEXTIN_SET:
        return _textin_filename_set(self);
    }
    return 0;
}
// #endregion
//-----------------------------------------------------------------------------
//    Emulator Objective-C Class
//-----------------------------------------------------------------------------
@implementation n80config

-(id)initWithN80:(n80em*)n80em config:(n80config_data*)config {
    if (self = [super init]) {
        self->n80em = n80em;
        self->data  = config;
        n80device.param.interlace       = config.interlace;
        n80device.param.use_new_pcg     = config.chiqlappe;
        n80device.vrtc.crt_hires_mode   = config.hires_crt ? _CRT_HIRES : _CRT_NORMAL;
        self->n80em.n80kb.tebkeyless    = config.tenkeyless;
        self->n80em.nowait              = config.nowait;
    }
    return self;
}

-(n80config const *)getParam {
    return (n80config const *)self->data;
}

-(void)enter_config {
    tmp_save_screen_mode   = n80device.crtc._screen_mode;
    tmp_save_text_visible  = n80device.crtc._text_visible;
    tmp_save_use_new_pcg   = n80device.param.use_new_pcg;
    tmp_tenkeyless         = self->n80em.n80kb.tebkeyless;
    tmp_z80_nowait         = self->n80em.nowait;

    [self->n80em.n80gr chg_bltfunc:3];
    n80device.crtc._text_visible = 0;	// text visible
    n80device.param.use_new_pcg = FALSE;// disable new pcg

    for (int r = 0; r < 25; ++r) {
        for (int c = 0; c < 80; ++c) {
            BYTE atr = 0x23;
            if ((r == 4 || r == 5) && (c > 0 && c < 79))
                atr = 0x27;
            else if (r == 7 && (c > 0 && c < 79))
                atr = 0x26;
            else if ((r > 8 && r < 24) && (c > 0 && c < 79))
                atr = 0x07;
            m_cui_attr[r][c] = atr;
            m_cui_text[r][c] = 32;
        }
    }
    item_cnt = 0;
    cur_idx = 0;
    const char* title1 = "    \xea\xeaN80\xea\xea PC-8001 Emulator for macOS (14.0[Sonoma] or later)";
    const char* title2 = "    Copyright 1993-2024 (C) H.Kanemoto, All Right Reserved.";
    const char* title3 = "System Config";
    const char* title4 = "usage:  [ESC] Quit, [\x1e\x1f] Move, [SP] Toggle, [\x1d\x1c] Value, [BS] Del-String";
    const char* title5 = "hint: ";

    args[0].org = &self->data->slot[0][0]; //self->n80em->m_n80_rom_file_name;
    args[1].org = &self->data->slot[1][0];//self->n80em->m_4th_rom_file_name;
    args[2].org = &self->data->slot[2][0];//self->n80em->m_fdu_rom_file_name;
    for (int r = 0; items[r].type != TYPE_TERMINATOR; ++r) {
        ++item_cnt;
        items[r].controller(&items[r], VERB_INIT, NULL);
        [self drawitem:&items[r]  line:r selected:r == 0];
    }
    memcpy(&m_cui_text[1][2], title1, strlen(title1));
    memcpy(&m_cui_text[2][2], title2, strlen(title2));
    memcpy(&m_cui_text[4][2], title3, strlen(title3));
    memcpy(&m_cui_text[5][2], title4, strlen(title4));
    memcpy(&m_cui_text[7][2], title5, strlen(title5));

    [self->n80em.n80gr draw_config:m_cui_text attr:m_cui_attr];
}

-(void)leave_config {
    n80device.crtc._screen_mode  = tmp_save_screen_mode;
    n80device.crtc._text_visible = tmp_save_text_visible;
    n80device.param.use_new_pcg  = tmp_save_use_new_pcg;
    self->n80em.n80kb.tebkeyless = tmp_tenkeyless;
    self->n80em.nowait           = tmp_z80_nowait;

    self->data.interlace  = n80device.param.interlace;
    self->data.chiqlappe  = n80device.param.use_new_pcg;
    self->data.hires_crt  = n80device.vrtc.crt_hires_mode == _CRT_HIRES;
    self->data.tenkeyless = self->n80em.n80kb.tebkeyless;
    self->data.nowait     = self->n80em.nowait;

    [self->n80em.n80gr chg_bltfunc:tmp_save_screen_mode];
    [self reflect_menu_state];
    
}

-(void)reflect_menu_state {
    AppDelegate *app = [[NSApplication sharedApplication] delegate];
    if (n80device.param.interlace)
        [[app menuInterlace] setState:NSControlStateValueOn];
    else
        [[app menuInterlace] setState:NSControlStateValueOff];
    if (self->n80em.n80kb.tebkeyless)
        [[app menuTenKeyLess] setState:NSControlStateValueOn];
    else
        [[app menuTenKeyLess] setState:NSControlStateValueOff];
    if (n80device.pcg_mode != 0)
        [[app menuPCG] setState:NSControlStateValueOn];
    else
        [[app menuPCG] setState:NSControlStateValueOff];
    if (n80device.param.use_new_pcg)
        [[app menuNewPCG] setState:NSControlStateValueOn];
    else
        [[app menuNewPCG] setState:NSControlStateValueOff];
}

-(void)OnKeyDown:(BYTE)key code:(GCKeyCode)c {
    if (c == GCKeyCodeLeftShift || c == GCKeyCodeRightShift) {
        keyevt.SHIFT = true;
        return;
    }
    keyevt.key = c;
    keyevt.code = key;
    keyevt.ascii = to_ascii(&keyevt);

    if (items[cur_idx].controller(&items[cur_idx], VERB_EVENT, &keyevt) == 1) {
        [self drawitem:&items[cur_idx] line:cur_idx selected:true];
        [self->n80em.n80gr draw_config:m_cui_text attr:m_cui_attr];

    } else if ((keyevt.key == GCKeyCodeTab && !keyevt.SHIFT) || keyevt.key == GCKeyCodeDownArrow) { // [TAB] or [DOWN]
        if (item_cnt > 1) {
            items[cur_idx].controller(&items[cur_idx], VERB_INIT, NULL);
            [self drawitem:&items[cur_idx] line:cur_idx selected:false];
            if (++cur_idx >= item_cnt)
                cur_idx = 0;
            [self drawitem:&items[cur_idx] line:cur_idx selected:true];
            [self->n80em.n80gr draw_config:m_cui_text attr:m_cui_attr];
        }
    } else if ((keyevt.key == GCKeyCodeTab && keyevt.SHIFT) || keyevt.key == GCKeyCodeUpArrow) { // [UP]
        if (item_cnt > 1) {
            items[cur_idx].controller(&items[cur_idx], VERB_INIT, NULL);
            [self drawitem:&items[cur_idx] line:cur_idx selected:false];
            if (--cur_idx < 0)
                cur_idx = item_cnt - 1;
            [self drawitem:&items[cur_idx] line:cur_idx selected:true];
            [self->n80em.n80gr draw_config:m_cui_text attr:m_cui_attr];
        }
    }
}

-(bool)OnKeyUp:(BYTE)key code:(GCKeyCode)c {
    if (c == GCKeyCodeLeftShift || c == GCKeyCodeRightShift) {
        keyevt.SHIFT = false;
        return false;
    } else if (c == GCKeyCodeEscape) {
        return true;
    }
    keyevt.key = c;
    keyevt.code = key;
    keyevt.ascii = to_ascii(&keyevt);
    return false;
}

-(void)drawitem:(LPVOID)_item line:(int)line selected:(bool)selected {
    LPITEM item = (LPITEM)_item;
    int row = line + 9;
    item->controller(item, selected ? VERB_PRINT_SEL : VERB_PRINT, &m_cui_text[row][1]);
    for (int i = 1; i < 79; ++i)
        m_cui_attr[row][i] = selected ? 0x22 : 0x07;
    if (selected) {
        char buff[128];
        utf8_to_sjis(item->hint, buff, 128);
        memset(&m_cui_text[7][1], 32, 78);
        //sprintf(&m_cui_text[7][1], " hint: %s", item->hint);
        sprintf(&m_cui_text[7][1], " hint: %s", buff);
    }
}
@end
//-----------------------------------------------------------------------------
//    Emulator Objective-C Class
//-----------------------------------------------------------------------------
@implementation n80config_data
-(id)init {
    if (self = [super init]) {
        memset(self->slot, 0, sizeof(self->slot));
    }
    return self;
}
-(void)set_char:(char)c index:(int)index slot:(int)slot {
    self->slot[slot][index] = (char)c;
}
-(char)get_char:(int)index slot:(int)slot {
    return self->slot[slot][index];
}
-(int)get_size:(int)slot {
    return (int)strlen(&self->slot[slot][0]);
}

@end
