//
//  n80em.h
//
//  Copyright (C) 1993-2016 Hiroshi Kanemoto.
//    Released under the MIT license
//    http://opensource.org/licenses/mit-license.php
//
//     Created:    2012/08/22    H.Kanemoto
//    Modified:    2016/04/05    H.Kanemoto    ported from MaxOS X
//
#ifndef __N80EM_H__
#define __N80EM_H__
//#if TARGET_OS_IPHONE
//#import <UIKit/UIKit.h>
//#endif
//#if TARGET_OS_OSX
#import <AppKit/AppKit.h>
//#endif
#import <Foundation/Foundation.h>
#import <AudioUnit/AudioUnit.h>
#import "DblBuffBmpImage.h"
#import <stdatomic.h>
#include "n80.h"
#include "pc-8001.h"
#include "libkern/OSAtomic.h"
//#if TARGET_OS_IPHONE
//#include "bytecoder.h"
//#endif

typedef void (^MFICallback)(void);
typedef void (^HaltCallback)(int32_t halt);
//-----------------------------------------------------------------------------
//    external variables/functions
//-----------------------------------------------------------------------------
extern void        _n80_device_initialize(LPN80DEVICE, SYNCFNC enter, SYNCFNC leave);
extern BYTE        _n80_mbread(USHORT Adr, DWORD *W);
extern void        _n80_mbwrite(USHORT Adr, BYTE v, DWORD *W);
extern BYTE        _n80_ioread(USHORT Port);
extern void        _n80_iowrite(USHORT Port, BYTE d);
extern void        __z80_set_external_method(LPZ80CPU cpu, LPZ80FNCMBR mbr, LPZ80FNCMBW mbw, LPZ80FNCIOR ior, LPZ80FNCIOW iow);
extern void        onkey_down(BYTE code);
extern void        onkey_up(BYTE code);
//-----------------------------------------------------------------------------
//    Emulator Class
//-----------------------------------------------------------------------------
@class n80sound;
@class n80graph;
@class n80keyboard;
@class n80config;
@class n80config_data;

@interface n80em : NSObject {
    @private
    DblBuffBmpImage     *dblbmp;
    NSConditionLock     *pauseZ80sync;
    volatile DWORD      state;
    n80sound            *sound;
    n80graph            *graph;
    n80keyboard         *keyboard;
    n80config           *config;
    atomic_int          request_cpu_signal;
}
@property (assign) BOOL main_rom_ready;
@property (assign) BOOL running;
@property (assign) BOOL pause;
@property (assign) BOOL terminating;
@property (assign) BOOL config_mode;
@property (assign) BOOL nowait;
@property (nonatomic, assign, readonly, getter=getN80) N80LX const *n80;
@property (nonatomic, assign, readonly, getter=getN80Sound) n80sound const *n80snd;
@property (nonatomic, assign, readonly, getter=getN80Graph) n80graph const *n80gr;
@property (nonatomic, assign, readonly, getter=getN80Keyboard) n80keyboard const *n80kb;
@property (nonatomic, assign, readonly, getter=getN80Config) n80config const *n80cfg;
@property (nonatomic, assign, readonly, getter=getBmpimg) DblBuffBmpImage const *bmpimg;
@property (nonatomic) MFICallback mfi_callback;
@property (nonatomic) HaltCallback halt_callback;

#if TARGET_OS_IPHONE
+(bytecoder*)loadFile:(NSString*)fullpath;
#endif

-(id)init:(DblBuffBmpImage*)bmp data:(n80config_data*)data;
-(void)start;
-(void)suspend;
-(void)resume;
-(void)enter_z80_thread;
-(void)key_clear;
-(void)req_dispose;
-(void)req_reset;
-(void)req_toggle_pcg;
-(void)req_eject:(int)unit;
#if TARGET_OS_OSX
-(void)req_open_image_file;
-(BOOL)req_load_main_rom;
-(BOOL)req_load_disk_rom;
-(BOOL)req_open_d88_file:(int)unit;
-(void)req_enter_config;
-(void)req_leave_config;
#else
-(void)reqOpenFile:(bytecoder*)data isN80:(BOOL)flag;
#endif
-(void)refleshScreen;

-(void)enterPause;
-(void)leavePause:(BOOL)terminating;

@end
#endif//__N80EM_H__
