//
//  n80keyboard.m
//  N80X
//
//  Created by hiroshi on 2021/08/21.
//
#import "n80em.h"
#import "n80keyboard.h"
#import "n80config.h"
//-----------------------------------------------------------------------------
//    Emulator C Functions
//-----------------------------------------------------------------------------
extern N80LX        n80lx;
static BYTE          down_matorix[80][2] =
{
    {0,0xfe}, {0,0xfd}, {0,0xfb}, {0,0xf7},
    {0,0xef}, {0,0xdf}, {0,0xbf}, {0,0x7f},
    {1,0xfe}, {1,0xfd}, {1,0xfb}, {1,0xf7},
    {1,0xef}, {1,0xdf}, {1,0xbf}, {1,0x7f},
    {2,0xfe}, {2,0xfd}, {2,0xfb}, {2,0xf7},
    {2,0xef}, {2,0xdf}, {2,0xbf}, {2,0x7f},
    {3,0xfe}, {3,0xfd}, {3,0xfb}, {3,0xf7},
    {3,0xef}, {3,0xdf}, {3,0xbf}, {3,0x7f},
    {4,0xfe}, {4,0xfd}, {4,0xfb}, {4,0xf7},
    {4,0xef}, {4,0xdf}, {4,0xbf}, {4,0x7f},
    {5,0xfe}, {5,0xfd}, {5,0xfb}, {5,0xf7},
    {5,0xef}, {5,0xdf}, {5,0xbf}, {5,0x7f},
    {6,0xfe}, {6,0xfd}, {6,0xfb}, {6,0xf7},
    {6,0xef}, {6,0xdf}, {6,0xbf}, {6,0x7f},
    {7,0xfe}, {7,0xfd}, {7,0xfb}, {7,0xf7},
    {7,0xef}, {7,0xdf}, {7,0xbf}, {7,0x7f},
    {8,0xfe}, {8,0xfd}, {8,0xfb}, {8,0xf7},
    {8,0xef}, {8,0xdf}, {8,0xbf}, {8,0x7f},
    {9,0xfe}, {9,0xfd}, {9,0xfb}, {9,0xf7},
    {9,0xef}, {9,0xdf}, {9,0xbf}, {9,0x7f}
};
static BYTE          up_matorix[80][2] =
{
    {0,0x01}, {0,0x02}, {0,0x04}, {0,0x08},
    {0,0x10}, {0,0x20}, {0,0x40}, {0,0x80},
    {1,0x01}, {1,0x02}, {1,0x04}, {1,0x08},
    {1,0x10}, {1,0x20}, {1,0x40}, {1,0x80},
    {2,0x01}, {2,0x02}, {2,0x04}, {2,0x08},
    {2,0x10}, {2,0x20}, {2,0x40}, {2,0x80},
    {3,0x01}, {3,0x02}, {3,0x04}, {3,0x08},
    {3,0x10}, {3,0x20}, {3,0x40}, {3,0x80},
    {4,0x01}, {4,0x02}, {4,0x04}, {4,0x08},
    {4,0x10}, {4,0x20}, {4,0x40}, {4,0x80},
    {5,0x01}, {5,0x02}, {5,0x04}, {5,0x08},
    {5,0x10}, {5,0x20}, {5,0x40}, {5,0x80},
    {6,0x01}, {6,0x02}, {6,0x04}, {6,0x08},
    {6,0x10}, {6,0x20}, {6,0x40}, {6,0x80},
    {7,0x01}, {7,0x02}, {7,0x04}, {7,0x08},
    {7,0x10}, {7,0x20}, {7,0x40}, {7,0x80},
    {8,0x01}, {8,0x02}, {8,0x04}, {8,0x08},
    {8,0x10}, {8,0x20}, {8,0x40}, {8,0x80},
    {9,0x01}, {9,0x02}, {9,0x04}, {9,0x08},
    {9,0x10}, {9,0x20}, {9,0x40}, {9,0x80}
};
//-----------------------------------------------------------------------------
//    Emulator Objective-C Class
//-----------------------------------------------------------------------------
@implementation n80keyboard

-(id)initWithN80:(n80em*)n80em {
    if (self = [super init]) {
        self->n80em = n80em;
        self.tebkeyless = true;
    }
    return self;
}

-(bool)toggle_tenkeyless {
    self.tebkeyless = !self.tebkeyless;
    [self->n80em.n80cfg reflect_menu_state];
    return self.tebkeyless;
}

-(void)set_tenkeyless:(bool)flag {
    self.tebkeyless = flag;
}

-(void)OnKeyDown:(BYTE)key code:(GCKeyCode)c {
    if (self->n80em.config_mode) {
        [self->n80em.n80cfg OnKeyDown:key code:c];
    } else {
        if (key < 0x50) {
            int  index = down_matorix[key][0];
            BYTE bit   = down_matorix[key][1];
            n80device.io.keys[index] &= bit;
        }
    }
}

-(void)OnKeyUp:(BYTE)key  code:(GCKeyCode)c {
    if (self->n80em.config_mode) {
        if ([self->n80em.n80cfg OnKeyUp:key code:c])
            [self->n80em req_leave_config];

    } else {
        if (key < 0x50) {
            int  index = up_matorix[key][0];
            BYTE bit   = up_matorix[key][1];
            n80device.io.keys[index] |= bit;
        } else if (key == 0x80) {
            [self->n80em req_reset];
    #if TARGET_OS_OSX
        } else if (key == 0x81) {
            [self->n80em req_open_image_file];
        } else if (key == 0x83) {
            [self->n80em req_enter_config];
    #endif
        } else if (key == 0x82) {
            [self->n80em req_toggle_pcg];
        }
    }
}

-(void)kb_keydown:(GCKeyCode)keycode  API_AVAILABLE(ios(14.0)){
    if (!self->n80em.main_rom_ready)
        return;
    BYTE key = [self keycode2key:keycode];
    if (key != 0xff) {
        [self OnKeyDown:key code:keycode];
    }
}

-(void)kb_keyup:(GCKeyCode)keycode  API_AVAILABLE(ios(14.0)){
    if (!self->n80em.main_rom_ready)
        return;
    BYTE key = [self keycode2key:keycode];
    if (key != 0xff) {
        [self OnKeyUp:key code:keycode];
    }
}

-(BYTE)keycode2key:(GCKeyCode)keycode  API_AVAILABLE(ios(14.0)){
#if TARGET_OS_IPHONE
  if (@available(iOS 14.0, *)) {
#endif
    if (keycode == GCKeyCodeKeypad0)        return 0x00;
    else if (keycode == GCKeyCodeKeypad1)   return 0x01;
    else if (keycode == GCKeyCodeKeypad2)   return 0x02;
    else if (keycode == GCKeyCodeKeypad3)   return 0x03;
    else if (keycode == GCKeyCodeKeypad4)   return 0x04;
    else if (keycode == GCKeyCodeKeypad5)   return 0x05;
    else if (keycode == GCKeyCodeKeypad6)   return 0x06;
    else if (keycode == GCKeyCodeKeypad7)   return 0x07;
    else if (keycode == GCKeyCodeKeypad8)   return 0x08;
    else if (keycode == GCKeyCodeKeypad9)   return 0x09;
    else if (keycode == GCKeyCodeKeypadAsterisk)    return 0x0a;
    else if (keycode == GCKeyCodeKeypadPlus)        return 0x0b;
    else if (keycode == GCKeyCodeKeypadEqualSign)   return 0x0c;
    else if (keycode == GCKeyCodeKeypadPeriod)      return 0x0e;
    else if (keycode == GCKeyCodeKeypadEnter)       return 0x0f;
    else if (keycode == GCKeyCodeReturnOrEnter)     return 0x0f;
    else if (keycode == GCKeyCodeKeyA)   return 0x11;
    else if (keycode == GCKeyCodeKeyB)   return 0x12;
    else if (keycode == GCKeyCodeKeyC)   return 0x13;
    else if (keycode == GCKeyCodeKeyD)   return 0x14;
    else if (keycode == GCKeyCodeKeyE)   return 0x15;
    else if (keycode == GCKeyCodeKeyF)   return 0x16;
    else if (keycode == GCKeyCodeKeyG)   return 0x17;
    else if (keycode == GCKeyCodeKeyH)   return 0x18;
    else if (keycode == GCKeyCodeKeyI)   return 0x19;
    else if (keycode == GCKeyCodeKeyJ)   return 0x1a;
    else if (keycode == GCKeyCodeKeyK)   return 0x1b;
    else if (keycode == GCKeyCodeKeyL)   return 0x1c;
    else if (keycode == GCKeyCodeKeyM)   return 0x1d;
    else if (keycode == GCKeyCodeKeyN)   return 0x1e;
    else if (keycode == GCKeyCodeKeyO)   return 0x1f;
    else if (keycode == GCKeyCodeKeyP)   return 0x20;
    else if (keycode == GCKeyCodeKeyQ)   return 0x21;
    else if (keycode == GCKeyCodeKeyR)   return 0x22;
    else if (keycode == GCKeyCodeKeyS)   return 0x23;
    else if (keycode == GCKeyCodeKeyT)   return 0x24;
    else if (keycode == GCKeyCodeKeyU)   return 0x25;
    else if (keycode == GCKeyCodeKeyV)   return 0x26;
    else if (keycode == GCKeyCodeKeyW)   return 0x27;
    else if (keycode == GCKeyCodeKeyX)   return 0x28;
    else if (keycode == GCKeyCodeKeyY)   return 0x29;
    else if (keycode == GCKeyCodeKeyZ)   return 0x2a;
    else if (keycode == GCKeyCodeOpenBracket)       return 0x10;
    else if (keycode == GCKeyCodeBackslash)         return 0x2d;
    else if (keycode == GCKeyCodeNonUSBackslash)    return 0x3f;
    else if (keycode == GCKeyCodeCloseBracket)      return 0x2b;
    else if (keycode == GCKeyCodeQuote)             return 0x3a;
    else if (keycode == GCKeyCodeEqualSign)         return 0x2e;
    else if (keycode == GCKeyCodeHyphen)            return 0x2f;
    else if (keycode == GCKeyCodeGraveAccentAndTilde)   return 0x2e;
    else if (keycode == GCKeyCodeZero)  return 0x30;
    else if (keycode == GCKeyCodeOne)   return 0x31;
    else if (keycode == GCKeyCodeTwo)   return 0x32;
    else if (keycode == GCKeyCodeThree) return 0x33;
    else if (keycode == GCKeyCodeFour)  return 0x34;
    else if (keycode == GCKeyCodeFive)  return 0x35;
    else if (keycode == GCKeyCodeSix)   return 0x36;
    else if (keycode == GCKeyCodeSeven) return 0x37;
    else if (keycode == GCKeyCodeEight) return 0x38;
    else if (keycode == GCKeyCodeNine)  return 0x39;
    else if (keycode == GCKeyCodeSemicolon)     return 0x3b;
    else if (keycode == GCKeyCodeComma)         return 0x3c;
    else if (keycode == GCKeyCodePeriod)        return 0x3d;
    else if (keycode == GCKeyCodeSlash)         return 0x3e;
    else if (keycode == GCKeyCodeNonUSPound)    return 0x2d;
    else if (keycode == GCKeyCodeHome)          return 0x40;
    else if (keycode == GCKeyCodeUpArrow)       return self.tebkeyless ? 0x08 : 0x41;
    else if (keycode == GCKeyCodeDownArrow)     return self.tebkeyless ? 0x02 : 0x41;
    else if (keycode == GCKeyCodeLeftArrow)     return self.tebkeyless ? 0x04 : 0x42;
    else if (keycode == GCKeyCodeRightArrow)    return self.tebkeyless ? 0x06 : 0x42;
    else if (keycode == GCKeyCodeInsert)        return 0x43;
    else if (keycode == GCKeyCodeDeleteOrBackspace) return 0x43;
    else if (keycode == GCKeyCodeLeftShift)     return 0x46;
    else if (keycode == GCKeyCodeRightShift)    return 0x46;
    else if (keycode == GCKeyCodeLeftControl)   return 0x47;
    else if (keycode == GCKeyCodeRightControl)  return 0x47;
    else if (keycode == GCKeyCodeTab)           return 0x48;
    else if (keycode == GCKeyCodeF1)            return 0x49;
    else if (keycode == GCKeyCodeF2)            return 0x4a;
    else if (keycode == GCKeyCodeF3)            return 0x4b;
    else if (keycode == GCKeyCodeF4)            return 0x4c;
    else if (keycode == GCKeyCodeF5)            return 0x4d;
    else if (keycode == GCKeyCodeF7)            return 0x83;    //[F7] = config
    else if (keycode == GCKeyCodeF9)            return 0x81;    //[F9] = open file
    else if (keycode == GCKeyCodeF11)           return 0x82;    //[F11] = pcg on/off
    else if (keycode == GCKeyCodeF12)           return 0x80;    //[F12] = reset
    else if (keycode == GCKeyCodeSpacebar)      return 0x4e;
    else if (keycode == GCKeyCodeEscape)        return 0x4f;
    else if (keycode == GCKeyCodeInternational1)    return 0x3f;    // "_"
    else if (keycode == GCKeyCodeInternational3)    return 0x2c;    // "¥"
    else if (keycode == GCKeyCodeLANG1)             return 0x45;    // kana -> kana
    else if (keycode == GCKeyCodeLANG2)             return 0x44;    // en-num -> GRPH
    else if (keycode == GCKeyCodeInternational2)    {NSLog(@"Intl2"); return 0xff;}
    else if (keycode == GCKeyCodeInternational4)    {NSLog(@"Intl4"); return 0xff;}
    else if (keycode == GCKeyCodeInternational5)    {NSLog(@"Intl5"); return 0xff;}
    else if (keycode == GCKeyCodeInternational6)    {NSLog(@"Intl6"); return 0xff;}
    else if (keycode == GCKeyCodeInternational7)    {NSLog(@"Intl7"); return 0xff;}
    else if (keycode == GCKeyCodeInternational8)    {NSLog(@"Intl8"); return 0xff;}
    else if (keycode == GCKeyCodeInternational9)    {NSLog(@"Intl9"); return 0xff;}
    else return 0xff;
#if TARGET_OS_IPHONE
  } else {
    return 0xff;
  }
#endif
}

@end
