//
//  n80em.h
//
//  Copyright (C) 1993-2019 Hiroshi Kanemoto.
//    Released under the MIT license
//    http://opensource.org/licenses/mit-license.php
//
//     Created:    2019/05/12    H.Kanemoto
//
#ifndef __N80_SOUND_H__
#define __N80_SOUND_H__
#import <Foundation/Foundation.h>
#import <AudioUnit/AudioUnit.h>
#import <sys/time.h>
#import "n80em.h"
#include "n80.h"
#include "pc-8001.h"
#include "libkern/OSAtomic.h"
//-----------------------------------------------------------------------------
//    Sound data structures
//-----------------------------------------------------------------------------
#define MAXSTsARRAYS    (8000)
#define    WAVETOP            (4000)
#define WAVEBOTTOM        (-4000)

typedef struct _sound_ticks_tag {
    double        ticks;    // T-states累計
    DWORD        state;    // 0:off 1:on, PCGの場合はサンプル数(周波数)
}    SAMPLETs, *LPSAMPLETs;
typedef struct _sound_ticks_array_tag {
    SAMPLETs    arr[MAXSTsARRAYS];
    double        ticks_sum;
    int            count;
}    SAMPLETsARR, *LPSAMPLETsARR;
typedef struct _sound_bank_tag {
    SAMPLETs    current;
    SAMPLETsARR    banks[2];
    int            bank;
}    SAMPLETsBANK, *LPSAMPLETsBANK;

//-----------------------------------------------------------------------------
//    Sound Class
//-----------------------------------------------------------------------------
@interface n80sound : NSObject {
    @private
    AudioUnit            gOutputUnit;
    n80em                *n80em;
    NSRecursiveLock        *sync;
}
-(id)initWithN80:(n80em*)n80em;
-(void)suspend;
-(BOOL)resume;
-(void)cpu_reset;
-(void)reqDispose;
-(void)callback:(UInt32)inNumberFrames ioData:(AudioBufferList *)ioData;
-(void)beep_on;
-(void)beep_off;
-(void)sound_on:(USHORT)freq ch:(int)ch;
-(void)sound_off;
-(void)append_sound_ticks:(DWORD)T;
-(void)lock;
-(void)unlock;

@end
#endif//__N80_SOUND_H__
